## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Implementaion of miscellaneous game functions

import CvUtil
from CvPythonExtensions import *
import CvEventInterface
import CustomFunctions
import ScenarioFunctions

# globals
cf = CustomFunctions.CustomFunctions()
gc = CyGlobalContext()
sf = ScenarioFunctions.ScenarioFunctions()

class CvGameUtils:
	"Miscellaneous game functions"
	def __init__(self): 
		pass
	
	def isVictoryTest(self):
		if ( gc.getGame().getElapsedGameTurns() > 10 ):
			return True
		else:
			return False

	def isVictory(self, argsList):
		eVictory = argsList[0]
		return True

	def isPlayerResearch(self, argsList):
		ePlayer = argsList[0]
		return True

	def getExtraCost(self, argsList):
		ePlayer = argsList[0]
		return 0

	def createBarbarianCities(self):
		return False
		
	def createBarbarianUnits(self):
		return False
		
	def skipResearchPopup(self,argsList):
		ePlayer = argsList[0]
		return False
		
	def showTechChooserButton(self,argsList):
		ePlayer = argsList[0]
		return True

	def getFirstRecommendedTech(self,argsList):
		ePlayer = argsList[0]
		return TechTypes.NO_TECH

	def getSecondRecommendedTech(self,argsList):
		ePlayer = argsList[0]
		eFirstTech = argsList[1]
		return TechTypes.NO_TECH
	
	def canRazeCity(self,argsList):
		iRazingPlayer, pCity = argsList
		return True
	
	def canDeclareWar(self,argsList):
		iAttackingTeam, iDefendingTeam = argsList
		return True
	
	def skipProductionPopup(self,argsList):
		pCity = argsList[0]
		return False
		
	def showExamineCityButton(self,argsList):
		pCity = argsList[0]
		return True

	def getRecommendedUnit(self,argsList):
		pCity = argsList[0]
		return UnitTypes.NO_UNIT

	def getRecommendedBuilding(self,argsList):
		pCity = argsList[0]
		return BuildingTypes.NO_BUILDING

	def updateColoredPlots(self):
		return False

	def isActionRecommended(self,argsList):
		pUnit = argsList[0]
		iAction = argsList[1]
		return False

	def unitCannotMoveInto(self,argsList):
		ePlayer = argsList[0]		
		iUnitId = argsList[1]
		iPlotX = argsList[2]
		iPlotY = argsList[3]
		return False

	def cannotHandleAction(self,argsList):
		pPlot = argsList[0]
		iAction = argsList[1]
		bTestVisible = argsList[2]
		return False

	def canBuild(self,argsList):
		iX, iY, iBuild, iPlayer = argsList
		pPlayer = gc.getPlayer(iPlayer)
		eTeam = gc.getTeam(pPlayer.getTeam())
		if pPlayer.isHuman() == False:
			# build farms instead of cottages when running aristo
			if iBuild == gc.getInfoTypeForString('BUILD_COTTAGE'):
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CONSTRUCTION')) == True :
					if pPlayer.getCivics(gc.getInfoTypeForString('CIVICOPTION_GOVERNMENT')) == gc.getInfoTypeForString('CIVIC_ARISTOCRACY'): 
						return 0
			# AI can't use those
			if iBuild == gc.getInfoTypeForString('BUILD_FORT'):
				return 0
			# Waste for elven civs to build these
			if iBuild == gc.getInfoTypeForString('BUILD_LUMBERMILL'):
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR') or pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
					return 0
			# Not even at guilds a good build for civs that use food
			if iBuild == gc.getInfoTypeForString('BUILD_WORKSHOP'):
				if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FALLOW')) :
					return 0
			# Luchuirp need fire for blasting workshops, but only one
			if gc.getBuildInfo(iBuild).getType()[0:11] == 'BUILD_MANA_':
				if pPlayer.isHasTech(gc.getInfoTypeForString('TECH_ELEMENTALISM')):
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
						if not pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_FIRE')):
							if iBuild != gc.getInfoTypeForString('BUILD_MANA_FIRE'):
								return 0
				# these are the civs that like to spectre spam, one ench node then they build the tower of necromancy then only death nodes
				if pPlayer.getCivilizationType() in (gc.getInfoTypeForString('CIVILIZATION_SHEAIM'), gc.getInfoTypeForString('CIVILIZATION_CALABIM')):
					if iBuild not in (gc.getInfoTypeForString('BUILD_MANA_DEATH'),gc.getInfoTypeForString('BUILD_MANA_CHAOS'),gc.getInfoTypeForString('BUILD_MANA_ENTROPY'),gc.getInfoTypeForString('BUILD_MANA_SHADOW'),gc.getInfoTypeForString('BUILD_MANA_ENCHANTMENT')):
						return 0
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_CHAOS')):
						if iBuild == gc.getInfoTypeForString('BUILD_MANA_CHAOS'):
							return 0
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_SHADOW')) or pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_COUNCIL_OF_ESUS')):
						if iBuild == gc.getInfoTypeForString('BUILD_MANA_SHADOW'):
							return 0
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_ENTROPY')) or pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL')) :
						if iBuild == gc.getInfoTypeForString('BUILD_MANA_ENTROPY'):
							return 0
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_ENCHANTMENT')):
						if iBuild == gc.getInfoTypeForString('BUILD_MANA_ENCHANTMENT'):
							return 0
				#one enchantment node for everyone
				if pPlayer.isHasTech(gc.getInfoTypeForString('TECH_ALTERATION')):
					if not pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_ENCHANTMENT')):
						if iBuild != gc.getInfoTypeForString('BUILD_MANA_ENCHANTMENT'):
							return 0
					elif iBuild == gc.getInfoTypeForString('BUILD_MANA_ENCHANTMENT'):
						return 0
		return -1	# Returning -1 means ignore; 0 means Build cannot be performed; 1 or greater means it can

	def cannotFoundCity(self,argsList):
		iPlayer, iPlotX, iPlotY = argsList
		pPlot = CyMap().plot(iPlotX,iPlotY)
		return False

	def cannotSelectionListMove(self,argsList):
		pPlot = argsList[0]
		bAlt = argsList[1]
		bShift = argsList[2]
		bCtrl = argsList[3]
		return False

	def cannotSelectionListGameNetMessage(self,argsList):
		eMessage = argsList[0]
		iData2 = argsList[1]
		iData3 = argsList[2]
		iData4 = argsList[3]
		iFlags = argsList[4]
		bAlt = argsList[5]
		bShift = argsList[6]
		return False

	def cannotDoControl(self,argsList):
		eControl = argsList[0]
		return False

	def canResearch(self,argsList):
		ePlayer = argsList[0]
		eTech = argsList[1]
		bTrade = argsList[2]
		return False

	def cannotResearch(self,argsList):
		ePlayer = argsList[0]
		eTech = argsList[1]
		bTrade = argsList[2]
		pPlayer = gc.getPlayer(ePlayer)
		eTeam = gc.getTeam(pPlayer.getTeam())
		
		if eTech == gc.getInfoTypeForString('TECH_ORDERS_FROM_HEAVEN'):
			if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_1):
				return True
			if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
				return True
			if pPlayer.isHuman() == False:
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
					return True
		if eTech == gc.getInfoTypeForString('TECH_WAY_OF_THE_EARTHMOTHER'):
			if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_3):
				return True
			if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
				return True
			if pPlayer.isHuman() == False:
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
					return True
		if eTech == gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS'):
			if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_0):
				return True
			if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
				return True
		if eTech == gc.getInfoTypeForString('TECH_MESSAGE_FROM_THE_DEEP'):
			if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_2):
				return True
			if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
				return True
			if pPlayer.isHuman() == False:
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
					return True
		if eTech == gc.getInfoTypeForString('TECH_CORRUPTION_OF_SPIRIT'):
			if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_4):
				return True
			if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
				return True
			if pPlayer.isHuman() == False:
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
					return True
		if eTech == gc.getInfoTypeForString('TECH_WAY_OF_THE_WISE'):
			if pPlayer.isHuman() == False:
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
					return True
		if eTech == gc.getInfoTypeForString('TECH_WAY_OF_THE_WICKED'):
			if pPlayer.isHuman() == False:
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
					return True
		if eTech == gc.getInfoTypeForString('TECH_HONOR'):
			if pPlayer.isHuman() == False:
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
					return True
		if eTech == gc.getInfoTypeForString('TECH_DECEPTION'):
			if pPlayer.isHuman() == False:
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
					return True

		if eTech == gc.getInfoTypeForString('TECH_SEAFARING'):
			if pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_LANUN'):
				return True
		#block useless techs
		if pPlayer.isHuman() == False:
			if eTech == gc.getInfoTypeForString('TECH_STIRRUPS'):
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_CLAN_OF_EMBERS'):
					return True	
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
					return True
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD'):
					return True
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
					if not pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_DEER')):
						return True
				elif not pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_HORSE')):
					if not pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_NIGHTMARE')):
						if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KURIOTATES'):
							return True
			if eTech == gc.getInfoTypeForString('TECH_RAGE'):
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
					return True	
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
					return True
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
					return True
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SHEAIM'):
					return True
			if eTech == gc.getInfoTypeForString('TECH_MITHRIL_WEAPONS'):
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SHEAIM'):
					return True
			if eTech == gc.getInfoTypeForString('TECH_PRECISION'): 
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
					return True
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD'):
					return True
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_DOVIELLO'):
					return True
			if eTech == gc.getInfoTypeForString('TECH_ANIMAL_MASTERY'): 
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
					return True
			if eTech == gc.getInfoTypeForString('TECH_PRIESTHOOD'):
				if pPlayer.getStateReligion() == -1: 
					if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
						return True
			#block opposed alignment node techs until the correct one has been researched
			if eTech == gc.getInfoTypeForString('TECH_DIVINATION'):
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
					if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_NECROMANCY')):
						return True
			if eTech == gc.getInfoTypeForString('TECH_NECROMANCY'):
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
					if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_DIVINATION')):
						return True
			#medicine is an expensive deadend without much gain except for rok civs and grigori
			if eTech == gc.getInfoTypeForString('TECH_MEDICINE'):
				if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_RUNES_OF_KILMORPH'):
					if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_GRIGORI'):
						bMedicineblocked = True
						for iTeam2 in range(gc.getMAX_TEAMS()):
							eTeam2 = gc.getTeam(iTeam2)
							if eTeam2.isAlive():
								if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_MEDICINE')):
									bMedicineblocked = False
						if bMedicineblocked:
							return True
						
		if CyGame().getWBMapScript():
			bBlock = sf.cannotResearch(ePlayer, eTech, bTrade)
			if bBlock:
				return True

		return False

	def canDoCivic(self,argsList):
		ePlayer = argsList[0]
		eCivic = argsList[1]
		return False

	def cannotDoCivic(self,argsList):
		ePlayer = argsList[0]
		eCivic = argsList[1]
		pPlayer = gc.getPlayer(ePlayer)
		eTeam = gc.getTeam(pPlayer.getTeam())
		
		if pPlayer.isHuman() == False :
			#AI can't cope well with conquest
			if eCivic == gc.getInfoTypeForString('CIVIC_CONQUEST'):
				if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FALLOW')) :
					return True
			#useless civic
			if eCivic == gc.getInfoTypeForString('CIVIC_MERCANTILISM'):
				return True
			#force them out of getting killed by maintenance
			if eCivic == gc.getInfoTypeForString('CIVIC_DESPOTISM'):
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CARTOGRAPHY')):
					return True
			if eCivic == gc.getInfoTypeForString('CIVIC_GOD_KING') :
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CARTOGRAPHY')):
					if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KURIOTATES') :
						if pPlayer.getNumCities() >= 7:
							return True
			#force aristocracy for fin civs
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CODE_OF_LAWS')):
				if gc.getCivicInfo(eCivic).getCivicOptionType() == gc.getInfoTypeForString('CIVICOPTION_GOVERNMENT'):
					if eCivic != gc.getInfoTypeForString('CIVIC_ARISTOCRACY'):
						if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')) :
							return True
				#force synergy civics with aristo
				elif pPlayer.getCivics(gc.getInfoTypeForString('CIVICOPTION_GOVERNMENT')) == gc.getInfoTypeForString('CIVIC_ARISTOCRACY'):
					if gc.getCivicInfo(eCivic).getCivicOptionType() == gc.getInfoTypeForString('CIVICOPTION_ECONOMY'):
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CALENDAR')):
							if eCivic != gc.getInfoTypeForString('CIVIC_AGRARIANISM') or eCivic != gc.getInfoTypeForString('CIVIC_GUARDIAN_OF_NATURE'):
								return True
		return False
		
	def canTrain(self,argsList):
		pCity = argsList[0]
		eUnit = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		bIgnoreCost = argsList[4]
		bIgnoreUpgrades = argsList[5]
		return False

	def cannotTrain(self,argsList):
		pCity = argsList[0]
		eUnit = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		bIgnoreCost = argsList[4]
		bIgnoreUpgrades = argsList[5]
		ePlayer = pCity.getOwner()
		pPlayer = gc.getPlayer(ePlayer)
		eUnitClass = gc.getUnitInfo(eUnit).getUnitClassType()
		eTeam = gc.getTeam(pPlayer.getTeam())
		
		if pPlayer.isHuman() == False and bContinue == False:
			infoCiv = gc.getCivilizationInfo(pPlayer.getCivilizationType())
			if eUnitClass == gc.getInfoTypeForString('UNITCLASS_WARRIOR'):
				pPlot = pCity.plot()
				if pPlot.getNumUnits() >= 3:
					iTrainingYard = infoCiv.getCivilizationBuildings(gc.getInfoTypeForString('BUILDINGCLASS_TRAINING_YARD'))
					if iTrainingYard != -1:
						if pPlayer.canConstruct(iTrainingYard, False, False, False):
							return True
			if eUnitClass == gc.getInfoTypeForString('UNITCLASS_SCOUT'):
				if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_SCOUT')) >= 1:
					return True
				iHuntingLodge = infoCiv.getCivilizationBuildings(gc.getInfoTypeForString('BUILDINGCLASS_HUNTING_LODGE'))
				if iHuntingLodge != -1:
					if pPlayer.canConstruct(iHuntingLodge, False, False, False):
						return True
			if eUnit == gc.getInfoTypeForString('UNIT_DEMAGOG'):
				if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DEMAGOG')) > 6:
					return True
			#limit overexpansion pre-festivals
			if eUnit == gc.getInfoTypeForString('UNIT_SETTLER'):
				#don't build settlements for the AI, drawbacks are higher than benefits
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KURIOTATES'):
					if pPlayer.getNumCities() >= pPlayer.getMaxCities():
						return True
				if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_SETTLER')) > 0:
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_FESTIVALS')) == False:
						return True
				if pCity.getPopulation() < 4:
					return True
			#only spend hammers on religion if civ plans to follow them longterm
			if eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_EMPYREAN'):
				if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_EMPYREAN')) > 3:
					return True
				if not pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_THE_EMPYREAN')):
					if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_MALAKIM'):
						return True
			if eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_FELLOWSHIP_OF_LEAVES'):
				if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_FELLOWSHIP_OF_LEAVES')) > 3:
					return True
				if not pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_FELLOWSHIP_OF_LEAVES')) :
					if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
						return True
			if eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_OCTOPUS_OVERLORDS'):
				if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_OCTOPUS_OVERLORDS')) > 3:
					return True
				if not pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_OCTOPUS_OVERLORDS')) :
					if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LANUN'):
						return True
			if eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_RUNES_OF_KILMORPH'):
				if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_RUNES_OF_KILMORPH')) > 3:
					return True
				if not pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_RUNES_OF_KILMORPH')):
					if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD') :
						return True
			if eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_THE_ASHEN_VEIL'):
				if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_THE_ASHEN_VEIL')) > 3:
					return True
				if not pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL')) :
					if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SHEAIM') :
						return True
			if eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_THE_ORDER'):
				if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_THE_ORDER')) > 3:
					return True
				if not pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_THE_ORDER')) :
					if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_BANNOR') :
						return True
			if eUnit == gc.getInfoTypeForString('UNIT_AIRSHIP'):
				return True
			#unit is dominated by warriors/scouts even for varn
			if eUnit == gc.getInfoTypeForString('UNIT_LIGHTBRINGER'):
				return True
			#axeman is better, build some for flavor
			if eUnit == gc.getInfoTypeForString('UNIT_DWARVEN_SOLDIER_RUNES'):
				if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_DWARVEN_SOLDIER_RUNES')) > 2:
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
						return True
			if eUnit == gc.getInfoTypeForString('UNIT_DROWN'):
				if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_DROWN')) > 2:
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
						return True
			if eUnit == gc.getInfoTypeForString('UNIT_FAWN'):
				if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_FAWN')) > 2:
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
						return True
			#champion is better if available, build some for flavor
			if eUnit == gc.getInfoTypeForString('UNIT_CRUSADER'):
				if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_CRUSADER')) > 2:
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')): 
						return True	
			if eUnit == gc.getInfoTypeForString('UNIT_PARAMANDER'):
				if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_PARAMANDER')) > 2:
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')): 
						return True
			if eUnit == gc.getInfoTypeForString('UNIT_MONK'):
				if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_MONK')) > 2:
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
						return True
			if eUnit == gc.getInfoTypeForString('UNIT_JAVELIN_THROWER'):
				if 2 * pPlayer.getNumCities() <= pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_ARCHER')) :
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
						return True
				if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_COPPER')):
					return True
			# not enough pop to feed large amounts of vampires
			if eUnit == gc.getInfoTypeForString('UNIT_VAMPIRE'):
				if pPlayer.getNumCities() <= pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_CHAMPION')) :
					return True
			# don't go pirating if there are big ships hunting you
			if eUnit == gc.getInfoTypeForString('UNIT_PRIVATEER'):
				iPrivateercounter = 0
				for iTeam2 in range(gc.getMAX_PLAYERS()):
					eTeam2 = gc.getTeam(iTeam2)
					if eTeam2.isAlive():
						if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_OPTICS')) and eTeam2.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')):
							iPrivateercounter = iPrivateercounter +1
				if iPrivateercounter > 1: 
					return True
			# don't start producing units that need to be finished fast in backwater cities
			if isWorldUnitClass(eUnitClass):
				if pCity.findBaseYieldRateRank(YieldTypes.YIELD_PRODUCTION)>2 and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
					if pCity.getUnitProduction(eUnit)==0:
						return True
			
		if eUnit == gc.getInfoTypeForString('UNIT_BEAST_OF_AGARES'):
			if pCity.getPopulation() <= 5:
				return True

		if pPlayer.getCivics(gc.getInfoTypeForString('CIVICOPTION_CULTURAL_VALUES')) == gc.getInfoTypeForString('CIVIC_CRUSADE'):
			if eUnit == gc.getInfoTypeForString('UNIT_WORKER'):
				return True
			if eUnit == gc.getInfoTypeForString('UNIT_SETTLER'):
				return True
			if eUnit == gc.getInfoTypeForString('UNIT_WORKBOAT'):
				return True

		if eUnit == gc.getInfoTypeForString('UNIT_ACHERON'):
			if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_ACHERON):
				return True

		if eUnit == gc.getInfoTypeForString('UNIT_DUIN'):
			if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_DUIN):
				return True
		
		# emergency warrior production
		if pPlayer.isHuman() == False:
			if eUnitClass != gc.getInfoTypeForString('UNITCLASS_WARRIOR'):
				if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_BARBARIAN')):
					if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_WARRIOR')) <= 1:
						if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
							if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCHERY')):
								return True
								
		if CyGame().getWBMapScript():
			bBlock = sf.cannotTrain(pCity, eUnit, bContinue, bTestVisible, bIgnoreCost, bIgnoreUpgrades)
			if bBlock:
				return True

		return False

	def canConstruct(self,argsList):
		pCity = argsList[0]
		eBuilding = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		bIgnoreCost = argsList[4]
		return False

	def cannotConstruct(self,argsList):
		pCity = argsList[0]
		eBuilding = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		bIgnoreCost = argsList[4]
		pPlayer = gc.getPlayer(pCity.getOwner())
		iBuildingClass = gc.getBuildingInfo(eBuilding).getBuildingClassType()
		eTeam = gc.getTeam(pPlayer.getTeam())
		
		#(almost) completely useless buildings for infernals
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_INFERNAL') and pPlayer.isHuman() == False :
			if eBuilding == gc.getInfoTypeForString('BUILDING_AQUEDUCT'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_PUBLIC_BATHS'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_HERBALIST'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_GRANARY'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_SMOKEHOUSE'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_INFIRMARY'):
				return True
				
		if pPlayer.getCivics(gc.getInfoTypeForString('CIVICOPTION_CULTURAL_VALUES')) == gc.getInfoTypeForString('CIVIC_CRUSADE'):
			if eBuilding == gc.getInfoTypeForString('BUILDING_ELDER_COUNCIL'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_MARKET'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_MONUMENT'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_MONEYCHANGER'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_THEATRE'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_AQUEDUCT'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_PUBLIC_BATHS'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_HERBALIST'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_CARNIVAL'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_COURTHOUSE'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_GAMBLING_HOUSE'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_GRANARY'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_SMOKEHOUSE'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_LIBRARY'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_HARBOR'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_ALCHEMY_LAB'):
				return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_BREWERY'):
				return True
		# some rule for proper building construction
		if pPlayer.isHuman() == False :
			if eBuilding == gc.getInfoTypeForString('BUILDING_MONUMENT'):
				if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_CREATIVE')) and pCity.getPopulation() <=5:
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_MONEYCHANGER'):
				if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')) and pCity.getPopulation() <=5:
					return True
				if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')) == False: 
					if pCity.getPopulation() <=8:
						return True
					if eTeam.getAtWarCount(True) >= 1:
						return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_TAX_OFFICE'):
				if pCity.getPopulation() <=12 or pCity.happyLevel() - pCity.unhappyLevel(1) <=0:
					return True
				if eTeam.getAtWarCount(True) >= 1:
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_THEATRE'):
				if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_CREATIVE')) and pCity.getPopulation() <=6:
					return True
				if eTeam.getAtWarCount(True) >= 1:
					return True
				if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_CREATIVE')) == False: 
					if pCity.getPopulation() <=12:
						return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_AQUEDUCT'):
				if pCity.badHealth(False)-pCity.goodHealth() < -1 :
					return True
				if eTeam.getAtWarCount(True) >= 1:
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_HERBALIST'):
				if pCity.badHealth(False)-pCity.goodHealth() < -1 :
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_CARNIVAL'):
				if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_CREATIVE')) :
					if pCity.happyLevel() - pCity.unhappyLevel(1) >0:
						return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_BREEDING_PIT'):
				if pCity.happyLevel() - pCity.unhappyLevel(1) <=0:
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_RELIQUARY'):
				if pCity.getPopulation() <=8:
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_INFIRMARY'):
				if pCity.badHealth(False)-pCity.goodHealth() < 1 :
					return True
				if eTeam.getAtWarCount(True) >= 1:
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_INN'):
				if pCity.getPopulation() <=10:
					return True
				if eTeam.getAtWarCount(True) >= 1:
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_TAVERN'):
				if pCity.getPopulation() <=15:
					return True
				if eTeam.getAtWarCount(True) >= 1:
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_ALCHEMY_LAB'):
				if pCity.getPopulation() <=10:
					return True
				if eTeam.getAtWarCount(True) >= 1:
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_PLANAR_GATE'):
				if CyGame().getGlobalCounter() <=30:
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_DEMONS_ALTAR'):
				if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_INFERNAL'):
					return True
			if eBuilding == gc.getInfoTypeForString('BUILDING_CITY_OF_A_THOUSAND_SLUMS'):
				if pCity.getPopulation() <=20:
					return True
			if 	eBuilding == gc.getInfoTypeForString('BUILDING_STABLE'):
				if pPlayer.getNumCities() >= 3:
					if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_STABLE')) * 3 > pPlayer.getNumCities():
						if pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_HIPPUS'):
							return True
			if 	eBuilding == gc.getInfoTypeForString('BUILDING_HUNTING_LODGE'):
				if pPlayer.getNumCities() >= 3:
					if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_HUNTING_LODGE')) * 3 >  pPlayer.getNumCities():
						if pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
							return True
			if 	eBuilding == gc.getInfoTypeForString('BUILDING_TRAINING_YARD'):
				if pPlayer.getNumCities() >= 3:
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR') or pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_HIPPUS') or pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
						if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_TRAINING_YARD')) * 3 >  pPlayer.getNumCities():
							return True
			if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_WARRIOR')) <= 3:
				if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
					if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCHERY')):
						return True
			if isWorldWonderClass(iBuildingClass):
				if pCity.findBaseYieldRateRank(YieldTypes.YIELD_PRODUCTION)>5 and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
					if pCity.getBuildingProduction(eBuilding)==0:
						return True
					
		if eBuilding == gc.getInfoTypeForString('BUILDING_SHRINE_OF_THE_CHAMPION'):
			iHero = cf.getHero(pPlayer)
			if iHero == -1:
				return True
			if CyGame().isUnitClassMaxedOut(iHero, 0) == False:
				return True
			if pPlayer.getUnitClassCount(iHero) > 0:
				return True

		if eBuilding == gc.getInfoTypeForString('BUILDING_MERCURIAN_GATE'):
			if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_HYBOREM_OR_BASIUM):
				return True
			if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL'):
				return True
			if pCity.isCapital():
				return True
			if pPlayer.isHuman() == False:
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
					return True

		iAltar1 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR')
		iAltar2 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_ANOINTED')
		iAltar3 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_BLESSED')
		iAltar4 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_CONSECRATED')
		iAltar5 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_DIVINE')
		iAltar6 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_EXALTED')
		iAltar7 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_FINAL')
		if (eBuilding == iAltar1 or eBuilding == iAltar2 or eBuilding == iAltar3 or eBuilding == iAltar4 or eBuilding == iAltar5 or eBuilding == iAltar6 or eBuilding == iAltar7):
			if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
				return True
			if eBuilding == iAltar1:
				if (pPlayer.countNumBuildings(iAltar2) > 0 or pPlayer.countNumBuildings(iAltar3) > 0 or pPlayer.countNumBuildings(iAltar4) > 0 or pPlayer.countNumBuildings(iAltar5) > 0 or pPlayer.countNumBuildings(iAltar6) > 0 or pPlayer.countNumBuildings(iAltar7) > 0):
					return True
			if eBuilding == iAltar2:
				if (pPlayer.countNumBuildings(iAltar3) > 0 or pPlayer.countNumBuildings(iAltar4) > 0 or pPlayer.countNumBuildings(iAltar5) > 0 or pPlayer.countNumBuildings(iAltar6) > 0 or pPlayer.countNumBuildings(iAltar7) > 0):
					return True
			if eBuilding == iAltar3:
				if (pPlayer.countNumBuildings(iAltar4) > 0 or pPlayer.countNumBuildings(iAltar5) > 0 or pPlayer.countNumBuildings(iAltar6) > 0 or pPlayer.countNumBuildings(iAltar7) > 0):
					return True
			if eBuilding == iAltar4:
				if (pPlayer.countNumBuildings(iAltar5) > 0 or pPlayer.countNumBuildings(iAltar6) > 0 or pPlayer.countNumBuildings(iAltar7) > 0):
					return True
			if eBuilding == iAltar5:
				if (pPlayer.countNumBuildings(iAltar6) > 0 or pPlayer.countNumBuildings(iAltar7) > 0):
					return True
			if eBuilding == iAltar6:
				if pPlayer.countNumBuildings(iAltar7) > 0:
					return True

		if pPlayer.isHuman() == False:
			if eBuilding == gc.getInfoTypeForString('BUILDING_PROPHECY_OF_RAGNAROK'):
				if pPlayer.getAlignment() != gc.getInfoTypeForString('ALIGNMENT_EVIL'):
					return True

		if eBuilding == gc.getInfoTypeForString('BUILDING_SMUGGLERS_PORT'):
			if pPlayer.isSmugglingRing() == False:
				return True

		return False

	def canCreate(self,argsList):
		pCity = argsList[0]
		eProject = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		return False

	def cannotCreate(self,argsList):
		pCity = argsList[0]
		eProject = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		pPlayer = gc.getPlayer(pCity.getOwner())
		eTeam = gc.getTeam(pPlayer.getTeam())
		
		if eProject == gc.getInfoTypeForString('PROJECT_PURGE_THE_UNFAITHFUL'):
			if pPlayer.isHuman() == False:
				return True
			if pPlayer.getStateReligion() == -1:
				return True

		if eProject == gc.getInfoTypeForString('PROJECT_BIRTHRIGHT_REGAINED'):
			if not pPlayer.isFeatAccomplished(FeatTypes.FEAT_GLOBAL_SPELL):
				return True

		if eProject == gc.getInfoTypeForString('PROJECT_STIR_FROM_SLUMBER'):
			if pPlayer.getPlayersKilled() == 0:
				return True

		if eProject == gc.getInfoTypeForString('PROJECT_GENESIS'):
			if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_ILLIANS'):
				return True
		
		if eProject == gc.getInfoTypeForString('PROJECT_SAMHAIN'):
			if pPlayer.getNumCities() <= 3:
				return True
		
		if pPlayer.isHuman() == False:
			if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_WARRIOR')) <= 2:
				if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
					if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCHERY')):
						return True

		if pPlayer.isHuman() == False:
			if pCity.findBaseYieldRateRank(YieldTypes.YIELD_PRODUCTION)>2:
				return True
		
		return False

	def canMaintain(self,argsList):
		pCity = argsList[0]
		eProcess = argsList[1]
		bContinue = argsList[2]
		return False

	def cannotMaintain(self,argsList):
		pCity = argsList[0]
		eProcess = argsList[1]
		bContinue = argsList[2]
		return False
    
	def AI_chooseTech(self,argsList):
		ePlayer = argsList[0]
		bFree = argsList[1]
		pPlayer = gc.getPlayer(ePlayer)
		pCity = pPlayer.getCapitalCity()
		eTeam = gc.getTeam(pPlayer.getTeam())
		iTech = -1
		print ("\n\nDoing tech AI for %s" %(pPlayer.getName()))
		
		#mithril working is by far the most important tier4 tech for most civs
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_TAXATION')) or pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')) :
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_PHILOSOPHY')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')):
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_FANATICISM')):
					if not pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MITHRIL')):
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MITHRIL_WORKING')) == False:
							if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
								if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
									if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
										iTech = gc.getInfoTypeForString('TECH_MITHRIL_WORKING')
		
		#grab the free artist
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_FESTIVALS')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_DRAMA')) == False:
			if CyGame().getSorenRandNum(100, "DRAMABEELINE") <= 20:
				bDramavalid=True
				for iTeam2 in range(gc.getMAX_TEAMS()):
					eTeam2 = gc.getTeam(iTeam2)
					if eTeam2.isAlive():
						if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_DRAMA')):
							bDramavalid = False
				if bDramavalid:
					iTech = gc.getInfoTypeForString('TECH_DRAMA')
		
		#grab the free commander
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_PHILOSOPHY')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_WARFARE')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_MILITARY_STRATEGY')) == False:
			if CyGame().getSorenRandNum(100, "MILSCIBEELINE") <= 20:
				bMilvalid=True
				for iTeam2 in range(gc.getMAX_TEAMS()):
					eTeam2 = gc.getTeam(iTeam2)
					if eTeam2.isAlive():
						if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_MILITARY_STRATEGY')):
							bMilvalid = False
				if bMilvalid:
					iTech = gc.getInfoTypeForString('TECH_MILITARY_STRATEGY')
		
		#grab the free sage
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCANE_LORE')) == False:
			if CyGame().getSorenRandNum(100, "ARCLOREBEELINE") <= 20:
				bArcLorevalid=True
				for iTeam2 in range(gc.getMAX_TEAMS()):
					eTeam2 = gc.getTeam(iTeam2)
					if eTeam2.isAlive():
						if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_ARCANE_LORE')):
							bArcLorevalid = False
				if bArcLorevalid:
					iTech = gc.getInfoTypeForString('TECH_ARCANE_LORE')
		
		#prioritize alignment specific node techs
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_KNOWLEDGE_OF_THE_ETHER')):
			if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_NECROMANCY')) or eTeam.isHasTech(gc.getInfoTypeForString('TECH_ELEMENTALISM')) or eTeam.isHasTech(gc.getInfoTypeForString('TECH_ALTERATION')) or eTeam.isHasTech(gc.getInfoTypeForString('TECH_DIVINATION')):
				if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_ARCANE')) or pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_SUMMONER')):
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_TRADE')):
						if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
							iTech = gc.getInfoTypeForString('TECH_NECROMANCY')
						if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
							iTech = gc.getInfoTypeForString('TECH_DIVINATION')
		
		#beeline mages for arcane civs after economy is in advanced state
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_NECROMANCY')) or eTeam.isHasTech(gc.getInfoTypeForString('TECH_ELEMENTALISM')) or eTeam.isHasTech(gc.getInfoTypeForString('TECH_ALTERATION')) or eTeam.isHasTech(gc.getInfoTypeForString('TECH_DIVINATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')) == False:
			if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_ARCANE')) or pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_SUMMONER')):
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_TRADE')):
					iTech = gc.getInfoTypeForString('TECH_SORCERY')

		#beeline archmages for arcane civs after economy is in full swing
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCANE_LORE')):
			if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_ARCANE')) or pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_SUMMONER')):
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_TAXATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_ENGINEERING')):
					if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_STRENGTH_OF_WILL')):
						iTech = gc.getInfoTypeForString('TECH_STRENGTH_OF_WILL')
		
		#if the AI ever gets around to building the altar this is useful
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_RELIGIOUS_LAW')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_RIGHTEOUSNESS')) == False:
			iAltar5 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_DIVINE')
			if pPlayer.countNumBuildings(iAltar5) == 1:
				iTech = gc.getInfoTypeForString('TECH_RIGHTEOUSNESS')
				
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_RIGHTEOUSNESS')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_OMNISCIENCE')) == False :
			iAltar6 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_EXALTED')
			if pPlayer.countNumBuildings(iAltar6) == 1:
				iTech = gc.getInfoTypeForString('TECH_OMNISCIENCE')
		
		#luchuirp specific goes down the engineering line for gargoyles and picks up mages and blasting workshops
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WRITING')) :
				if CyGame().getSorenRandNum(100, "LUCHUIRPBEELINE") <= 50 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_ENGINEERING')) == False:				
					iTech = gc.getInfoTypeForString('TECH_ENGINEERING')
				elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_ELEMENTALISM')) == False:
					iTech = gc.getInfoTypeForString('TECH_ELEMENTALISM')
				elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')) == False:
					iTech = gc.getInfoTypeForString('TECH_SORCERY')
				elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_MACHINERY')) == False:
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BOWYERS')):
						iTech = gc.getInfoTypeForString('TECH_MACHINERY')
							
		#pickup iron working for civs that profit greatly from it
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_TAXATION')) or pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')) :
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_PHILOSOPHY')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')):
				if not pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_IRON')):
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_BALSERAPHS'):
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')) == False:
							iTech = gc.getInfoTypeForString('TECH_IRON_WORKING')
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_CLAN_OF_EMBERS'):
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')) == False:
							iTech = gc.getInfoTypeForString('TECH_IRON_WORKING')
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_DOVIELLO'):
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')) == False:
							iTech = gc.getInfoTypeForString('TECH_IRON_WORKING')
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_GRIGORI'):
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')) == False:
							iTech = gc.getInfoTypeForString('TECH_IRON_WORKING')
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_ILLIANS'):
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')) == False:
							iTech = gc.getInfoTypeForString('TECH_IRON_WORKING')
					
		#advanced religion beelines
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CORRUPTION_OF_SPIRIT')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_MALEVOLENT_DESIGNS')) == False:
			if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL'):
				if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_FANATICISM')):
						iTech = gc.getInfoTypeForString('TECH_MALEVOLENT_DESIGNS')
						
		
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ORDERS_FROM_HEAVEN')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) :
			if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_RIGHTEOUSNESS')) :
				if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ORDER'):
					if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_FANATICISM')):
							iTech = gc.getInfoTypeForString('TECH_RIGHTEOUSNESS')
							
		
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MESSAGE_FROM_THE_DEEP')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) :
			if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCANE_LORE')) :
				if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_OCTOPUS_OVERLORD'):
					if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_KNOWLEDGE OF THE ETHER')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_TRADE')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')):
							iTech = gc.getInfoTypeForString('TECH_ARCANE_LORE')
					elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_CODE_OF_LAWS')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_PRIESTHOOD')):
						iTech = gc.getInfoTypeForString('TECH_FANATICISM')

		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HONOR')) :
			if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_RELIGIOUS_LAW')) :
				if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_EMPYREAN'):
					if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_PRIESTHOOD')):
							iTech = gc.getInfoTypeForString('TECH_RELIGIOUS_LAW')
		
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS')) :
			if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_COMMUNE_WITH_NATURE')) :
				if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_FELLOWSHIP_OF_LEAVES'):
					if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_FERAL_BOND')):
							iTech = gc.getInfoTypeForString('TECH_COMMUNE_WITH_NATURE')
							
		#pickup advanced economy techs when not at war, currency has a lower priority compared to sanitation
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CURRENCY')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CODE_OF_LAWS')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_TRADE')):
			if eTeam.getAtWarCount(True) == 0:
				iTech = gc.getInfoTypeForString('TECH_CURRENCY') 
		#pickup economy techs that also double as military
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_TRADE')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SAILING')) or eTeam.isHasTech(gc.getInfoTypeForString('TECH_HORSEBACK_RIDING')) :
				iTech = gc.getInfoTypeForString('TECH_TRADE') 
		
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CONSTRUCTION')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')):
			if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
				if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
					iTech = gc.getInfoTypeForString('TECH_CONSTRUCTION')
			
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')):
			iTech = gc.getInfoTypeForString('TECH_SANITATION') 
		
		#pickup advanced economy techs when not at war
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MATHEMATICS')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CURRENCY')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')):
			if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')) :
				if eTeam.getAtWarCount(True) == 0:
					iTech = gc.getInfoTypeForString('TECH_MATHEMATICS') 
		
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_TAXATION')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CURRENCY')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_MATHEMATICS')):
			if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')) :
				if eTeam.getAtWarCount(True) == 0:
					iTech = gc.getInfoTypeForString('TECH_TAXATION') 
		
		# get the religion advanced techs when appropriate
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CORRUPTION_OF_SPIRIT')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_INFERNAL_PACT')) == False:
			if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL'):
				if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
					iTech = gc.getInfoTypeForString('TECH_INFERNAL_PACT')
		
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_EARTHMOTHER')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARETE')) == False:
			if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_RUNES_OF_KILMORPH'):
				if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
					iTech = gc.getInfoTypeForString('TECH_ARETE')
		
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_HIDDEN_PATHS')) == False:
			if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_FELLOWSHIP_OF_LEAVES'):
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR') or pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
					iTech = gc.getInfoTypeForString('TECH_HIDDEN_PATHS')
		
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MESSAGE_FROM_THE_DEEP')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_MIND_STAPLING')) == False:
			if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_OCTOPUS_OVERLORDS'):
				if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
					iTech = gc.getInfoTypeForString('TECH_MIND_STAPLING')
				elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')):
					iTech = gc.getInfoTypeForString('TECH_MIND_STAPLING')
					
					
		#start the Order beeline for Bannor earlier
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_BANNOR') and eTeam.isHasTech(gc.getInfoTypeForString('TECH_PHILOSOPHY')) == False:
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')):
				iTech = gc.getInfoTypeForString('TECH_PHILOSOPHY')
		
		#pickup bronzeworking at some point, since it is such a great tech
		if pPlayer.getNumCities() >= 4 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')) == False:
			iTech = gc.getInfoTypeForString('TECH_BRONZE_WORKING')
		
		#starts the AV beeline for sheaim earlier and Sheaim can defend very well with skeletons from adepts
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SHEAIM') and eTeam.isHasTech(gc.getInfoTypeForString('TECH_KNOWLEDGE_OF_THE_ETHER')) == False:
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MYSTICISM')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')):
				iTech = gc.getInfoTypeForString('TECH_KNOWLEDGE_OF_THE_ETHER')
		
		#if we can cast enchanted weapon with our adepts KoTE is great
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_KNOWLEDGE_OF_THE_ETHER')) == False:
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MYSTICISM')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')):
				if eTeam.getAtWarCount(True) > 0:
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_ENCHANTMENT')) :
						iTech = gc.getInfoTypeForString('TECH_KNOWLEDGE_OF_THE_ETHER')
		
		#pickup education after initial settler burst
		if pPlayer.getNumCities() >= 3 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) == False:
			iTech = gc.getInfoTypeForString('TECH_EDUCATION')
		
		#beeline trade if we get a trade monopoly
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_TRADE')) == False:
			if CyGame().getSorenRandNum(100, "TRADEBEELINE") <= 20:
				bTradevalid=True
				for iTeam2 in range(gc.getMAX_TEAMS()):
					eTeam2 = gc.getTeam(iTeam2)
					if eTeam2.isAlive():
						if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_TRADE')):
							bTradevalid = False
				if bTradevalid:
					iTech = gc.getInfoTypeForString('TECH_TRADE')
		
		
		#pickup improvement techs if we get a significant amount of them in our culture radius:
		if pPlayer.getNumCities() > 1:
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ANIMAL_HUSBANDRY')) == False or eTeam.isHasTech(gc.getInfoTypeForString('TECH_FISHING')) == False or eTeam.isHasTech(gc.getInfoTypeForString('TECH_HUNTING')) == False:
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_AGRICULTURE')):
					ilivestock = 0
					iseafoods = 0
					igame = 0
					for i in range (CyMap().numPlots()):
						pPlot = CyMap().plotByIndex(i)
						if pPlot.isOwned():
							if pPlot.getOwner() == ePlayer:
								eBonus = pPlot.getBonusType(TeamTypes.NO_TEAM)
								if eBonus == gc.getInfoTypeForString('BONUS_SHEEP') :
									ilivestock+=1
								if eBonus == gc.getInfoTypeForString('BONUS_COW') :
									ilivestock+=1
								if eBonus == gc.getInfoTypeForString('BONUS_PIG') :
									ilivestock+=1
								if eBonus == gc.getInfoTypeForString('BONUS_CLAM'):
									iseafoods +=1
								if eBonus == gc.getInfoTypeForString('BONUS_CRAB') :
									iseafoods += 1
								if eBonus == gc.getInfoTypeForString('BONUS_FISH') :
									iseafoods += 2
								if eBonus == gc.getInfoTypeForString('BONUS_DEER') :
									igame += 1
								if eBonus == gc.getInfoTypeForString('BONUS_IVORY') :
									igame += 2
								if eBonus == gc.getInfoTypeForString('BONUS_FUR') :
									igame += 2
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LANUN'):
					iseafoods += 5
				if	iseafoods >= 5 :
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_FISHING')) == False:
						iTech = gc.getInfoTypeForString('TECH_FISHING')
				if igame>=6:
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HUNTING')) == False:
						iTech = gc.getInfoTypeForString('TECH_HUNTING')
				if ilivestock>=3:
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ANIMAL_HUSBANDRY')) == False:
						iTech = gc.getInfoTypeForString('TECH_ANIMAL_HUSBANDRY')
		
		#beeline religion founding
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_PHILOSOPHY')) and pPlayer.getNumCities() >=3:
			if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')) :
				bOrdValid = True
				if pPlayer.countHolyCities() > 0 :
					bOrdValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
					bOrdValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD') :
					bOrdValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_MALAKIM'):
					bOrdValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LANUN') :
					bOrdValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SHEAIM'):	
					bOrdValid = False
				bEmpValid = True
				if pPlayer.countHolyCities() > 0 :
					bEmpValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR') :
					bEmpValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD') :
					bEmpValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_BANNOR'):
					bEmpValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LANUN') :
					bEmpValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SHEAIM'):
					bEmpValid = False
				for iTeam2 in range(gc.getMAX_TEAMS()):
					eTeam2 = gc.getTeam(iTeam2)
					if eTeam2.isAlive():
						if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_ORDERS_FROM_HEAVEN')):
							bOrdValid = False
						if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_HONOR')):
							bEmpValid = False
				if CyGame().getSorenRandNum(100, "Orderbeeline") <= 50:
					bOrdValid = False
				if CyGame().getSorenRandNum(100, "Empbeeline") <= 50:
					bEmpValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_BANNOR'):
					if pPlayer.countHolyCities() == 0 :
						bOrdValid = True
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_MALAKIM'):
					if pPlayer.countHolyCities() == 0 :
						bEmpValid = True
				if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_5):
					bEmpValid = False
				if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_1):
					bOrdValid = False
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
					bOrdValid = False
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
					bEmpValid = False
				if bEmpValid :
					if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_HONOR')):
						iTech = gc.getInfoTypeForString('TECH_HONOR')
				if bOrdValid :
					if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_ORDERS_FROM_HEAVEN')):
						iTech = gc.getInfoTypeForString('TECH_ORDERS_FROM_HEAVEN')
					
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_PHILOSOPHY')) or eTeam.isHasTech(gc.getInfoTypeForString('TECH_KNOWLEDGE_OF_THE_ETHER')):
			if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')) and pPlayer.getNumCities() >=3:
				bAshValid = True
				if pPlayer.countHolyCities() > 0 :
					bAshValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
					bAshValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD') :
					bAshValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_MALAKIM') :
					bAshValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_BANNOR') :
					bAshValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LANUN'):
					bAshValid = False
				bEsuValid = True
				if pPlayer.countHolyCities() > 0 :
					bEsuValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR') :
					bEsuValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD') :
					bEsuValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_MALAKIM') :
					bEsuValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_BANNOR'):
					bEsuValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LANUN') :
					bEsuValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SHEAIM'):	
					bEsuValid = False
				for iTeam2 in range(gc.getMAX_PLAYERS()):
					eTeam2 = gc.getTeam(iTeam2)
					if eTeam2.isAlive():
						if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_CORRUPTION_OF_SPIRIT')):
							bAshValid = False
						if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_DECEPTION')):
							bEsuValid = False
				if CyGame().getSorenRandNum(100, "Ashbeeline") <= 50:
					bAshValid = False
				if CyGame().getSorenRandNum(100, "Esusbeeline") <= 50:
					bEsuValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SHEAIM'):
					if pPlayer.countHolyCities() == 0 :
						bAshValid = True
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
					if pPlayer.countHolyCities() ==0 :
						bEsuValid = True
				if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_PHILOSOPHY')):
					bEsuValid = False
				if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_6) :
					bEsuValid = False
				if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_4) :
					bAshValid = False
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
					bAshValid = False
				if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
					bEsuValid = False
				if bEsuValid :
					if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_DECEPTION')):
						iTech = gc.getInfoTypeForString('TECH_DECEPTION')
				if bAshValid :
					if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_CORRUPTION_OF_SPIRIT')):
						iTech = gc.getInfoTypeForString('TECH_CORRUPTION_OF_SPIRIT')

		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MYSTICISM')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_MESSAGE_FROM_THE_DEEP')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_FISHING')) and pPlayer.getNumCities() >=3:
			if pPlayer.getAlignment() != gc.getInfoTypeForString('ALIGNMENT_GOOD'):
				if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
					bOctValid = True
					if pPlayer.countHolyCities() > 0 :
						bOctValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR') :
						bOctValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD') :
						bOctValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_BANNOR') :
						bOctValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_MALAKIM') :
						bOctValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SHEAIM'):	
						bOctValid = False
					for iTeam2 in range(gc.getMAX_PLAYERS()):
						eTeam2 = gc.getTeam(iTeam2)
						if eTeam2.isAlive():
							if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_MESSAGE_FROM_THE_DEEP')):
								bOctValid = False
					if CyGame().getSorenRandNum(100, "Octbeeline") <= 66:
						bOctValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LANUN'):
						if pPlayer.countHolyCities() == 0 :
							bOctValid = True
					if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_2):
						bOctValid = False
					if bOctValid == True:
						iTech=gc.getInfoTypeForString('TECH_MESSAGE_FROM_THE_DEEP')
	
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MYSTICISM')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_EARTHMOTHER')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_MINING')) and pPlayer.getNumCities() >=3:
			if pPlayer.getAlignment() != gc.getInfoTypeForString('ALIGNMENT_EVIL'):
				if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
					bRunesValid = True
					if pPlayer.countHolyCities() > 0 :
						bRunesValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
						bRunesValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LANUN'):
						bRunesValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_BANNOR'):
						bRunesValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_MALAKIM'):
						bRunesValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SHEAIM'):	
						bRunesValid = False
					for iTeam2 in range(gc.getMAX_PLAYERS()):
						eTeam2 = gc.getTeam(iTeam2)
						if eTeam2.isAlive():
							if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_EARTHMOTHER')):
								bRunesValid = False	
					if CyGame().getSorenRandNum(100, "Runesbeeline") <= 66:
						bRunesValid = False
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD'):
						if pPlayer.countHolyCities() == 0 :
							bRunesValid = True
					if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_3):
						bRunesValid = False
					if bRunesValid :
						iTech=gc.getInfoTypeForString('TECH_WAY_OF_THE_EARTHMOTHER')

		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MYSTICISM')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS')) == False and pPlayer.getNumCities() >=3:
			if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')) :
				bLeavesValid = True
				if pPlayer.countHolyCities() > 0 :
					bLeavesValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD'): 
					bLeavesValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LANUN') :
					bLeavesValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_BANNOR') :
					bLeavesValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_MALAKIM') :
					bLeavesValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SHEAIM'):	
					bLeavesValid = False
				for iTeam2 in range(gc.getMAX_PLAYERS()):
					eTeam2 = gc.getTeam(iTeam2)
					if eTeam2.isAlive():
						if eTeam2.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS')):
							bLeavesValid = False
				if CyGame().getSorenRandNum(100, "Leavesbeeline") <= 66:
					bLeavesValid = False
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
					if not pPlayer.countHolyCities() > 0 :
						bLeavesValid = True
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
					if not pPlayer.countHolyCities() > 0 :
						bLeavesValid = True
				if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_0):
					bLeavesValid = False
				if bLeavesValid :
					iTech=gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS')
		
		#calabim need their manors
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_CALABIM'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CODE_OF_LAWS')) == False:
				iTech = gc.getInfoTypeForString('TECH_CODE_OF_LAWS')
		
		#luchuirp need barnaxus and wood golems fast
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ANCIENT_CHANTS')) :
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CONSTRUCTION')) == False:
					iTech = gc.getInfoTypeForString('TECH_CONSTRUCTION')
		
		#hippus techs, horse centered
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_HIPPUS'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) :
				if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_HORSEBACK_RIDING')) :
					iTech = gc.getInfoTypeForString('TECH_HORSEBACK_RIDING')
				elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')):
					if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_WARHORSES')):
						iTech = gc.getInfoTypeForString('TECH_WARHORSES')
		
		#kurios synergize better with cottages than other civs and should get them early
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KURIOTATES'):
			if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) :
				iTech = gc.getInfoTypeForString('TECH_EDUCATION')
		
		#start the religion founding beelines and provides useful building/civic
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MYSTICISM'))== False: 
			iTech = gc.getInfoTypeForString('TECH_MYSTICISM')
		

		#even without calendar boni, every civ needs this early for agrarianism
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CALENDAR')) == False:
			iTech = gc.getInfoTypeForString('TECH_CALENDAR')
		
		#warrens for CoE
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_CLAN_OF_EMBERS'):
			if pPlayer.getNumCities() >=3:
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MASONRY')) == False:
					iTech = gc.getInfoTypeForString('TECH_MASONRY')
		
		#even without mining boni, every nonelvenciv needs this for mines and chopping
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MINING')) == False :
			if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
				if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
					iTech = gc.getInfoTypeForString('TECH_MINING')
		
		#the AI does not work well if it can't build roads
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_EXPLORATION')) == False:
			iTech = gc.getInfoTypeForString('TECH_EXPLORATION')
		
		#festivals is a great safety tech to prevent the AIs from bankrupting and markets are great for fin civs that don't need the protection
		if pPlayer.getNumCities() >=2 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CALENDAR')) == True and eTeam.isHasTech(gc.getInfoTypeForString('TECH_FESTIVALS'))== False: 
			iTech = gc.getInfoTypeForString('TECH_FESTIVALS')
		
		#another safety tech, however not as necessary, open borders are probably the bigger draw compared to city states
		if pPlayer.getNumCities() >= 5 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CARTOGRAPHY')) == False:
			iTech = gc.getInfoTypeForString('TECH_CARTOGRAPHY') 
		
		#start the aristocracy early for fin civs
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CODE_OF_LAWS')) == False :
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) :
				if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')) :
					iTech = gc.getInfoTypeForString('TECH_CODE_OF_LAWS')
		
		#Ljos specific stuff, mainly beeline to way of the forest
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS')): 
				if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCHERY')):
					iTech = gc.getInfoTypeForString('TECH_ARCHERY')
			elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')): 
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MYSTICISM')): 
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HUNTING')) == False:
						iTech = gc.getInfoTypeForString('TECH_HUNTING')
			elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_ANCIENT_CHANTS')):
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_AGRICULTURE')):
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CALENDAR')):
						iTech = gc.getInfoTypeForString('TECH_EDUCATION')
		
		#Svart specific stuff, similar to ljos. As the AI beelines poisons pretty early anyway, it does not need to be included
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ANIMAL_MASTERY')) == False:
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')): 
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_FERAL_BOND')): 
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_TAXATION')):  
							iTech = gc.getInfoTypeForString('TECH_ANIMAL_MASTERY')
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ANIMAL_HANDLING')) == False:
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HUNTING')): 
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')): 
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_TRADE')): 
							if pPlayer.getStateReligion() != -1: 
								iTech = gc.getInfoTypeForString('TECH_ANIMAL_HANDLING')
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HUNTING')) == False: 
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')): 
					iTech = gc.getInfoTypeForString('TECH_HUNTING')
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) == False:
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ANCIENT_CHANTS')):
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_AGRICULTURE')): 
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CALENDAR')):
							iTech = gc.getInfoTypeForString('TECH_EDUCATION')
		
		#if we have lots of floodplains cottaging them is great for the AI
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) == False:
			ifloodplain=0
			print ifloodplain
			for i in range (CyMap().numPlots()):
				pPlot = CyMap().plotByIndex(i)
				if not pPlot.isNone():
					if pPlot.isOwned():
						if pPlot.getOwner() == ePlayer:
							eFeature = pPlot.getFeatureType()
							if eFeature == gc.getInfoTypeForString('FEATURE_FLOOD_PLAINS'):
								ifloodplain= ifloodplain +1
			if ifloodplain>=3:
				iTech = gc.getInfoTypeForString('TECH_EDUCATION')
		
		#calc resources around the capital and tech accordingly
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HUNTING')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EXPLORATION')): 
			ihunting = 0
			for iI in range(1, 21):
				pLoopPlot = pCity.getCityIndexPlot(iI)
				if pLoopPlot:
					eBonus = pLoopPlot.getBonusType(TeamTypes.NO_TEAM)
					if eBonus != BonusTypes.NO_BONUS:
						if not pLoopPlot.isNone():
							if eBonus == gc.getInfoTypeForString('BONUS_FUR') :
								ihunting = ihunting +2
							if eBonus == gc.getInfoTypeForString('BONUS_DEER') :
								ihunting = ihunting +1
			if	ihunting >= 5 :
				iTech = gc.getInfoTypeForString('TECH_HUNTING')
						
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MINING')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CALENDAR')) == False:
			imining = 0
			icalendar = 0
			for iI in range(1, 21):
				pLoopPlot = pCity.getCityIndexPlot(iI)
				if pLoopPlot:
					eBonus = pLoopPlot.getBonusType(TeamTypes.NO_TEAM)
					if eBonus != BonusTypes.NO_BONUS:
						if not pLoopPlot.isNone():
							if eBonus == gc.getInfoTypeForString('BONUS_GOLD'):
								imining = imining +1
							if eBonus == gc.getInfoTypeForString('BONUS_GEMS') :
								imining = imining +1
							if eBonus == gc.getInfoTypeForString('BONUS_DYE') :
								icalendar = icalendar +1
							if eBonus == gc.getInfoTypeForString('BONUS_INCENSE') :
								icalendar = icalendar +1
			if	imining > 0 :
				if	imining >= icalendar:
					iTech = gc.getInfoTypeForString('TECH_MINING')
			if	icalendar > 0 :
				if	icalendar >= imining:
					iTech = gc.getInfoTypeForString('TECH_CALENDAR')

		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_AGRICULTURE')) == False:
			igrain = 0
			for iI in range(1, 21):
				pLoopPlot = pCity.getCityIndexPlot(iI)
				if pLoopPlot:
					eBonus = pLoopPlot.getBonusType(TeamTypes.NO_TEAM)
					if eBonus != BonusTypes.NO_BONUS:
						if not pLoopPlot.isNone():
							if eBonus == gc.getInfoTypeForString('BONUS_WHEAT'):
								igrain = igrain +3
							if eBonus == gc.getInfoTypeForString('BONUS_CORN') :
								igrain = igrain +3
							if eBonus == gc.getInfoTypeForString('BONUS_RICE') :
								igrain = igrain +3
					if not pLoopPlot.isNone():
						if pLoopPlot.isFreshWater():
							if not pLoopPlot.isHills():
								iFeature = pLoopPlot.getFeatureType()
								if not iFeature == gc.getInfoTypeForString('FEATURE_FOREST'):
									if not iFeature == gc.getInfoTypeForString('FEATURE_SCRUB'):
										igrain= igrain +1
			if	igrain >= 2 :	
				iTech = gc.getInfoTypeForString('TECH_AGRICULTURE')
				
				
		if pPlayer.countNumCoastalCities() >=1 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_FISHING')) == False: 
			iseafood = 0
			for iI in range(1, 21):
				pLoopPlot = pCity.getCityIndexPlot(iI)
				if pLoopPlot:
						eBonus = pLoopPlot.getBonusType(TeamTypes.NO_TEAM)
						if eBonus != BonusTypes.NO_BONUS:
							if not pLoopPlot.isNone():
								if eBonus == gc.getInfoTypeForString('BONUS_CLAM'):
									iseafood = iseafood +1
								if eBonus == gc.getInfoTypeForString('BONUS_CRAB') :
									iseafood = iseafood +1
								if eBonus == gc.getInfoTypeForString('BONUS_FISH') :
									iseafood = iseafood +2
				if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LANUN'):
					iseafood = iseafood +2
			if	iseafood >= 2 :
				iTech = gc.getInfoTypeForString('TECH_FISHING')
		
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CRAFTING')) == False: 
			inumwines = 0
			for iI in range(1, 21):
				pLoopPlot = pCity.getCityIndexPlot(iI)
				if pLoopPlot:
					eBonus = pLoopPlot.getBonusType(TeamTypes.NO_TEAM)
					if eBonus != BonusTypes.NO_BONUS:
						if not pLoopPlot.isNone():
							if eBonus == gc.getInfoTypeForString('BONUS_WINE') :
								inumwines = inumwines +1
			if	inumwines >= 2 :
				iTech = gc.getInfoTypeForString('TECH_CRAFTING')
		
		#if we get in an early war and have copper, beeline it for bronze warriors
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')) == False:
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MINING')):
				if eTeam.getAtWarCount(True) >= 1:
					bhascopper=0
					for i in range (CyMap().numPlots()):
						pPlot = CyMap().plotByIndex(i)
						if pPlot.isOwned():
							if pPlot.getOwner() == ePlayer:
								eBonus = pPlot.getBonusType(TeamTypes.NO_TEAM)
								if eBonus == gc.getInfoTypeForString('BONUS_COPPER') :
									bhascopper=1
					if bhascopper==1:
						iTech = gc.getInfoTypeForString('TECH_BRONZE_WORKING')
		
		if iTech != -1:
			if eTeam.isHasTech(iTech) == False:
				print iTech
				print ("Player %s Tech chosen is %s!"%(pPlayer.getName(),gc.getTechInfo(iTech).getType()))
				return iTech
		
		return TechTypes.NO_TECH

	def AI_chooseProduction(self,argsList):
		pCity = argsList[0]
		pPlayer = gc.getPlayer(pCity.getOwner())
		eTeam = gc.getTeam(pPlayer.getTeam())
		bOverride = false
		pPlot = pCity.plot()
		civtype = pPlayer.getCivilizationType()
		infoCiv = gc.getCivilizationInfo(civtype)
		iWorkshop =	gc.getInfoTypeForString('BUILDING_SIEGE_WORKSHOP')
		iBath = gc.getInfoTypeForString('BUILDING_PUBLIC_BATHS')
		iManor = gc.getInfoTypeForString('BUILDING_GOVERNORS_MANOR')
		iWarrens = gc.getInfoTypeForString('BUILDING_WARRENS')
		iBlastingWorkshop = gc.getInfoTypeForString('BUILDING_BLASTING_WORKSHOP')
		iMageGuild = gc.getInfoTypeForString('BUILDING_MAGE_GUILD')
		iAdept = gc.getInfoTypeForString('UNIT_ADEPT')
		iCatapult = gc.getInfoTypeForString('UNIT_CATAPULT')
		iWorkboat = gc.getInfoTypeForString('UNIT_WORKBOAT')
		iGranary = gc.getInfoTypeForString('BUILDING_GRANARY')
		iSmokehouse = gc.getInfoTypeForString('BUILDING_SMOKEHOUSE')
		iWorker = gc.getInfoTypeForString('UNIT_WORKER')
		iVeilTemple = gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_VEIL')
		iVeilDisciple = gc.getInfoTypeForString('UNIT_DISCIPLE_THE_ASHEN_VEIL')
		iOrderTemple = gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_ORDER')
		iOrderDisciple = gc.getInfoTypeForString('UNIT_DISCIPLE_THE_ORDER')
		iEmpyreanTemple = gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_EMPYREAN')
		iEmpyreanDisciple = gc.getInfoTypeForString('UNIT_DISCIPLE_EMPYREAN')
		iTowerElement = gc.getInfoTypeForString('BUILDING_TOWER_OF_THE_ELEMENTS')
		iTowerAlteration = gc.getInfoTypeForString('BUILDING_TOWER_OF_ALTERATION')
		iTowerNecromancy = gc.getInfoTypeForString('BUILDING_TOWER_OF_NECROMANCY')
		iTowerDivination = gc.getInfoTypeForString('BUILDING_TOWER_OF_DIVINATION')
		iLibrary = gc.getInfoTypeForString('BUILDING_LIBRARY')
		iNationalEpic = gc.getInfoTypeForString('BUILDING_NATIONAL_EPIC')
		iHeroicEpic= gc.getInfoTypeForString('BUILDING_HEROIC_EPIC')
		iShrineoftheChampion= gc.getInfoTypeForString('BUILDING_SHRINE_OF_THE_CHAMPION')
		iHuntingLodge = gc.getInfoTypeForString('BUILDING_HUNTING_LODGE')
		iArcheryRange = gc.getInfoTypeForString('BUILDING_ARCHERY_RANGE')
		iStable = gc.getInfoTypeForString('BUILDING_STABLE')
		iLeavesTemple =gc.getInfoTypeForString('BUILDING_TEMPLE_OF_LEAVES')
		iLeavesDisciple =gc.getInfoTypeForString('UNIT_DISCIPLE_FELLOWSHIP_OF_LEAVES')
		
		print("\n\nDoing AI for %s's City %s" %(pPlayer.getName(), pCity.getName()))
		
		#build the manors
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_CALABIM'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CODE_OF_LAWS')):
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_GOVERNORS_MANOR')) == 0:
					if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_ORGANIZED')) or pCity.getBaseYieldRate(1) >=6:
						if pPlot.getNumUnits() >= 2:
							pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iManor, -1, False, False, False, True)
							bOverride = true
							print ("Manor built")
							return bOverride
		
		#worker burst in the early turns to improve/chop; also more workers for elven civs
		if (eTeam.isHasTech(gc.getInfoTypeForString('TECH_MINING')) or pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR') or pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			iWorkers = 1
			print ("Worker 1")
			print iWorkers
			if eTeam.getAtWarCount(True) == 0:
				print ("Worker 2")
				print iWorkers
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_WARRENS')) == 1 or pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR') or pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
					iWorkers = iWorkers*1.7
					print ("Worker 3")
					print iWorkers
				if pPlayer.getNumCities() >=1:
					iWorkers = iWorkers * 14.0002 
					iGameTurn = CyGame().getGameTurn()
					if CyGame().getGameSpeedType() == gc.getInfoTypeForString('GAMESPEED_QUICK'):
						iGameTurn = iGameTurn * 3 / 2
					if CyGame().getGameSpeedType() == gc.getInfoTypeForString('GAMESPEED_EPIC'):
						iGameTurn = iGameTurn / 3 * 2
					if CyGame().getGameSpeedType() == gc.getInfoTypeForString('GAMESPEED_MARATHON'):
						iGameTurn = iGameTurn/ 3
					iDivisor = 6.0001+pPlayer.getNumCities()
					iDivisor = iDivisor +(iGameTurn/30)
					iWorkers = iWorkers/iDivisor
					print ("Worker new")
					print iWorkers
				if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
					print ("Worker 4")
					print iWorkers
					if pCity.getBaseYieldRate(1) >=7:
						print ("Worker 5")
						print iWorkers
						if iWorkers * pPlayer.getNumCities() > pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_WORKER')):
							print ("Worker 6")
							print iWorkers
							if pPlot.getNumUnits() >= 2:
								pCity.pushOrder(OrderTypes.ORDER_TRAIN, iWorker, -1, False, False, False, False)
								bOverride = true
								print iWorkers
								print ("Worker built")
								return bOverride
		
		# warrens
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_CLAN_OF_EMBERS'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MASONRY')):
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_WARRENS')) == 0:
					if pCity.getCulture(pCity.getOwner())>0: 
						if pCity.getBaseYieldRate(1) >=5:
							if pPlot.getNumUnits() >= 2:
								pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iWarrens, -1, False, False, False, True)
								bOverride = true
								print ("Warrens built")
								return bOverride
		#blasting workshops
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')):
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_SCULPTORS_STUDIO')) == 1 or eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')):
					if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_BLASTING_WORKSHOP')) == 0:
						if pCity.getCulture(pCity.getOwner())>0: 
							if pCity.getBaseYieldRate(1) >=7:
								if pPlot.getNumUnits() >= 2:
									pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iBlastingWorkshop, -1, False, False, False, False)
									bOverride = true
									print ("BlastingWorkshop built")
									return bOverride
		# hunting lodges for svarts
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HUNTING')):
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_HUNTING_LODGE')) == 0:
					if pCity.getBaseYieldRate(1) >=10:
						if pCity.getCulture(pCity.getOwner())>0: 
							if pPlot.getNumUnits() >= 2:
								pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iHuntingLodge, -1, False, False, False, False)
								bOverride = true
								print ("Hunting Lodge built")
								return bOverride
		#ranges for the ljos
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCHERY')):
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_ARCHERY_RANGE')) == 0:
					if pCity.getBaseYieldRate(1) >=10:
						if pCity.getCulture(pCity.getOwner())>0: 
							if pPlot.getNumUnits() >= 2:
								pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iArcheryRange, -1, False, False, False, False)
								bOverride = true
								print ("Archery Range built")
								return bOverride
		# and stables for the hippus
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_HIPPUS'):
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HORSEBACK_RIDING')):
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_STABLE')) == 0:
					if pCity.getBaseYieldRate(1) >=10:
						if pCity.getCulture(pCity.getOwner())>0: 
							if pPlot.getNumUnits() >= 2:
								pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iStable, -1, False, False, False, False)
								bOverride = true
								print ("Stable built")
								return bOverride
		#get some workshops built asap
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CONSTRUCTION')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
				if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
					if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_SIEGE_WORKSHOP')) == 0:
						if pPlayer.getNumCities() >= 4:
							if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_SIEGE_WORKSHOP')) * 4 <  pPlayer.getNumCities():
								if pCity.getBaseYieldRate(1) >=7:
									if pPlot.getNumUnits() >= 2:
										pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iWorkshop, -1, False, False, False, False)
										bOverride = true
										print ("Workshop built")
										return bOverride
		# get one mageguild built asap
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_KNOWLEDGE_OF_THE_ETHER')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_CLAN_OF_EMBERS'):
				if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_DOVIELLO'):
					if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_MAGE_GUILD')) == 0:
						if pPlayer.getNumCities() >= 4:
							if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_MAGE_GUILD'))* 6 <  pPlayer.getNumCities():
								if pCity.getBaseYieldRate(1) >=7:
									if pPlot.getNumUnits() >= 2:
										pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iMageGuild, -1, False, False, False, False)
										bOverride = true
										print ("Mage Guild built")
										return bOverride
		#get one military building in each city. some weighting for diversity
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')) or eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCHERY')) or eTeam.isHasTech(gc.getInfoTypeForString('TECH_HUNTING')) or eTeam.isHasTech(gc.getInfoTypeForString('TECH_HORSEBACK_RIDING')):
			iBuilding1 = infoCiv.getCivilizationBuildings(gc.getInfoTypeForString('BUILDINGCLASS_TRAINING_YARD'))
			iBuilding2 = infoCiv.getCivilizationBuildings(gc.getInfoTypeForString('BUILDINGCLASS_ARCHERY_RANGE'))
			iBuilding3 = infoCiv.getCivilizationBuildings(gc.getInfoTypeForString('BUILDINGCLASS_HUNTING_LODGE'))
			iBuilding4 = infoCiv.getCivilizationBuildings(gc.getInfoTypeForString('BUILDINGCLASS_STABLE'))
			iCount=0
			if pCity.getNumBuilding(iBuilding1)==1:
				iCount+=1
			if pCity.getNumBuilding(iBuilding2)==1:
				iCount+=1
			if pCity.getNumBuilding(iBuilding3)==1:
				iCount+=1
			if pCity.getNumBuilding(iBuilding4)==1:
				iCount+=1
			if iCount==0:
				iTrainingYard=0
				iArcheryRange=0
				iHuntingLodge=0
				iStable=0
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')):
					iTrainingYard=iTrainingYard+1000
					iTrainingYard=iTrainingYard+pPlayer.getBuildingClassCountPlusMaking(iBuilding2)*50
					iTrainingYard=iTrainingYard+pPlayer.getBuildingClassCountPlusMaking(iBuilding3)*50
					iTrainingYard=iTrainingYard+pPlayer.getBuildingClassCountPlusMaking(iBuilding4)*50
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')):
						iTrainingYard=iTrainingYard+1000
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MITHIRL')):
						iTrainingYard=iTrainingYard+600
					elif pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_IRON')):
						iTrainingYard=iTrainingYard+400
					elif pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_COPPER')):
						iTrainingYard=iTrainingYard+200
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_ENCHANTMENT')):
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_KNOWLEDGE_OF_THE_ETHER')):
							iTrainingYard=iTrainingYard+200
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD'):
						iTrainingYard=iTrainingYard+2000
					if pPlayer.getCivics(gc.getInfoTypeForString('CIVICOPTION_CULTURAL_VALUES')) == gc.getInfoTypeForString('CIVIC_NATIONHOOD'):
						iTrainingYard=iTrainingYard+200
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCHERY')):
					iArcheryRange=iArcheryRange+1000
					iArcheryRange=iArcheryRange+pPlayer.getBuildingClassCountPlusMaking(iBuilding1)*50
					iArcheryRange=iArcheryRange+pPlayer.getBuildingClassCountPlusMaking(iBuilding3)*50
					iArcheryRange=iArcheryRange+pPlayer.getBuildingClassCountPlusMaking(iBuilding4)*50
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BOWYERS')):
						iArcheryRange=iArcheryRange+1000
					if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_DEFENDER')):
						iArcheryRange=iArcheryRange+400
					if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_DEXTEROUS')):
						iArcheryRange=iArcheryRange+200
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_ENCHANTMENT')):
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')):
							iArcheryRange=iArcheryRange+200
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HUNTING')):
					iHuntingLodge=iHuntingLodge+1000
					iHuntingLodge=iHuntingLodge+pPlayer.getBuildingClassCountPlusMaking(iBuilding1)*50
					iHuntingLodge=iHuntingLodge+pPlayer.getBuildingClassCountPlusMaking(iBuilding2)*50
					iHuntingLodge=iHuntingLodge+pPlayer.getBuildingClassCountPlusMaking(iBuilding4)*50
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ANIMAL_HANDLING')):
						iHuntingLodge=iHuntingLodge+1000
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_POISONS')):
						iHuntingLodge=iHuntingLodge+200
					if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_SINISTER')):
						iHuntingLodge=iHuntingLodge+200
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_NATURE')):
						if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')):
							iHuntingLodge=iHuntingLodge+200
				if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HORSEBACK_RIDING')):
					iStable=iStable+1000
					iStable=iStable+pPlayer.getBuildingClassCountPlusMaking(iBuilding1)*50
					iStable=iStable+pPlayer.getBuildingClassCountPlusMaking(iBuilding2)*50
					iStable=iStable+pPlayer.getBuildingClassCountPlusMaking(iBuilding3)*50
					if eTeam.isHasTech(gc.getInfoTypeForString('TECH_STIRRUPS')):
						iStable=iStable+1000
					if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_HORSELORD')):
						iStable=iStable+200
					if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_RAIDERS')):
						iStable=iStable+200
					if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KURIOTATES'):
						iStable=iStable+400
				if pCity.getCulture(pCity.getOwner())>0:
						if pCity.getBaseYieldRate(1) >=8:
							if pPlot.getNumUnits() >= 2:
								if iTrainingYard>=iStable & iTrainingYard>=iHuntingLodge & iTrainingYard>=iArcheryRange and pCity.canConstruct(iBuilding1,False,False,False):
									pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iBuilding1, -1, False, False, False, True)
									bOverride = true
									print ("Training Yard built")
									return bOverride
								elif iArcheryRange>=iTrainingYard & iArcheryRange>=iHuntingLodge & iArcheryRange>=iStable and pCity.canConstruct(iBuilding2,False,False,False):
									pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iBuilding2, -1, False, False, False, True)
									bOverride = true
									print ("Archery Range built")
									return bOverride
								elif iHuntingLodge>=iTrainingYard & iHuntingLodge>=iArcheryRange & iHuntingLodge>=iStable and pCity.canConstruct(iBuilding3,False,False,False):
									pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iBuilding3, -1, False, False, False, True)
									bOverride = true
									print ("Hunting Lodge built")
									return bOverride
								elif iStable>=iTrainingYard & iStable>=iArcheryRange & iStable>=iHuntingLodge and pCity.canConstruct(iBuilding4,False,False,False):
									pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iBuilding4, -1, False, False, False, True)
									bOverride = true
									print ("Stable built")
									return bOverride
		#build a proper amount of adepts
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_KNOWLEDGE_OF_THE_ETHER')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("adept check1")
			iArcane = 0.25
			if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_ARCANE')):
				iArcane = iArcane+0.25
			if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_CHARISMATIC')):
				iArcane = iArcane+0.25
			if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_SUMMONER')):
				iArcane = iArcane+0.25
			if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')):
				iArcane = iArcane*4
			elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')) == False:
				if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_DEATH')):
					iArcane = iArcane*4
				elif pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_ENTROPY')):
					iArcane = iArcane +0.25
				elif pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_ENCHANTMENT')):
					iArcane = iArcane +0.25
				elif pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_BODY')):
					iArcane = iArcane +0.25
				elif pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_SHADOW')):
					iArcane = iArcane +0.25
			if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_CLAN_OF_EMBERS'):
				if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_INFERNAL'):
					if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_MAGE_GUILD')) == 1:
						if pCity.getBaseYieldRate(1) >=7:
							if iArcane * pPlayer.getNumCities() > 2 * pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_MAGE')):
								if iArcane * pPlayer.getNumCities() > 3 * pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_ADEPT')):
									if pPlot.getNumUnits() >= 2:
										pCity.pushOrder(OrderTypes.ORDER_TRAIN, iAdept, -1, False, False, False, False)
										bOverride = true
										print ("Adept built")
										return bOverride
		#if at war we need some catapults
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CONSTRUCTION')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("cata check1")
			if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_BLASTING_POWDER')):
				if eTeam.getAtWarCount(True) > 0:
					print ("warplan check1")
					if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
						if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
							if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD'):
								if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
									if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_SIEGE_WORKSHOP')) == 1:
										if pPlayer.getNumCities() > 5 * pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_CATAPULT')):
											if pCity.getBaseYieldRate(1) >=7:
												if pPlot.getNumUnits() >= 2:
													pCity.pushOrder(OrderTypes.ORDER_TRAIN, iCatapult, -1, False, False, False, False)
													bOverride = true
													print ("Catapult built")
													return bOverride
		#public baths are insanely good, should be built asap
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("pb check1")
			if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_INFERNAL'):
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_PUBLIC_BATHS')) == 0:
					if pCity.happyLevel() - pCity.unhappyLevel(1) <0:
						if pCity.getBaseYieldRate(1) >=5:
							if pPlot.getNumUnits() >= 2:
								pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iBath, -1, False, False, False, False)
								bOverride = true
								print ("Pbath built")
								return bOverride
		#fix for the -300 AIvalue weight modifier. built smoke and gran if lots of growth potential or health needed
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ANIMAL_HUSBANDRY')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN') and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_INFERNAL'):
			print ("smokehouse check1")
			if eTeam.getAtWarCount(True) == 0:
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_SMOKEHOUSE')) == 0:
					if pCity.getCulture(pCity.getOwner())>0: 
						if pCity.happyLevel() - pCity.unhappyLevel(1) >7:
							if pCity.foodDifference(false) >=4:
								if pCity.getBaseYieldRate(1) >=7:
									if pPlot.getNumUnits() >= 2:
										pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iSmokehouse, -1, False, False, False, False)
										bOverride = true
										print ("Smokehouse built")
										return bOverride
						elif pCity.goodHealth() <= pCity.badHealth(False):
							if pCity.happyLevel() - pCity.unhappyLevel(1) >0:
								if pPlayer.getNumAvailableBonuses(gc.getInfoTypeForString('BONUS_COW'))>0 or pPlayer.getNumAvailableBonuses(gc.getInfoTypeForString('BONUS_PIG'))>0 or pPlayer.getNumAvailableBonuses(gc.getInfoTypeForString('BONUS_SHEEP'))>0:
									if pCity.getBaseYieldRate(1) >=5:
										if pPlot.getNumUnits() >= 2:
											pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iSmokehouse, -1, False, False, False, False)
											bOverride = true
											print ("Smokehouse built")
											return bOverride
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_AGRICULTURE')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN') and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_INFERNAL'):
			print ("granary check1")
			if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_GRANARY')) == 0:
				if eTeam.getAtWarCount(True) == 0:
					if pCity.getCulture(pCity.getOwner())>0: 
						if pCity.happyLevel() - pCity.unhappyLevel(1) >7:
							if pCity.foodDifference(false) >=4:
								if pCity.getBaseYieldRate(1) >=7:
									if pPlot.getNumUnits() >= 2:
										pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iGranary, -1, False, False, False, False)
										bOverride = true
										print ("Granary built")
										return bOverride
						elif pCity.goodHealth() <= pCity.badHealth(False):
							if pCity.happyLevel() - pCity.unhappyLevel(1) >0:
								if pPlayer.getNumAvailableBonuses(gc.getInfoTypeForString('BONUS_WHEAT'))>0 or pPlayer.getNumAvailableBonuses(gc.getInfoTypeForString('BONUS_CORN'))>0 or pPlayer.getNumAvailableBonuses(gc.getInfoTypeForString('BONUS_RICE'))>0:
									if pCity.getBaseYieldRate(1) >=5:
										if pPlot.getNumUnits() >= 2:
											pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iGranary, -1, False, False, False, False)
											bOverride = true
											print ("Granary built")
											return bOverride
		#spread the overlords if has holy city
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MESSAGE_FROM_THE_DEEP')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("Octo1 check")
			if pCity.isHolyCityByType(gc.getInfoTypeForString('RELIGION_OCTOPUS_OVERLORDS')):
				if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_OCTOPUS_OVERLORDS'):
					if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
						if not pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
							if not pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
								if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_OVERLORDS')) == 0:
									if pCity.getBaseYieldRate(1) >=5:
										if pPlot.getNumUnits() >= 2:
											pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, (gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_OVERLORDS')), -1, False, False, False, False)
											bOverride = true
											return bOverride
											
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MESSAGE_FROM_THE_DEEP')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("veil disc check1")
			if pCity.isHolyCityByType(gc.getInfoTypeForString('RELIGION_OCTOPUS_OVERLORDS')):
				if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_OCTOPUS_OVERLORDS'):
					if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
						if not pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
							if not pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
								if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_OVERLORDS')) == 1:
									if pCity.getBaseYieldRate(1) >=5:
										if pPlot.getNumUnits() >= 2:
											pCity.pushOrder(OrderTypes.ORDER_TRAIN, (gc.getInfoTypeForString('UNIT_DISCIPLE_OCTOPUS_OVERLORDS')), -1, False, False, False, False)
											bOverride = true
											print ("veil disciple built")
											return bOverride
		#spread the veil if has holy city
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CORRUPTION_OF_SPIRIT')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("Veil1 check")
			if pCity.isHolyCityByType(gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL')):
				if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL'):
					if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
						if not pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
							if not pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
								if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_VEIL')) == 0:
									if pCity.getBaseYieldRate(1) >=5:
										if pPlot.getNumUnits() >= 2:
											pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iVeilTemple, -1, False, False, False, False)
											bOverride = true
											return bOverride
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CORRUPTION_OF_SPIRIT')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("veil disc check1")
			if pCity.isHolyCityByType(gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL')):
				if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL'):
					if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
						if not pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
							if not pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
								if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_VEIL')) == 1:
									if pCity.getBaseYieldRate(1) >=5:
										if pPlot.getNumUnits() >= 2:
											pCity.pushOrder(OrderTypes.ORDER_TRAIN, iVeilDisciple, -1, False, False, False, False)
											bOverride = true
											print ("veil disciple built")
											return bOverride
		#spread the order if has holy city
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ORDERS_FROM_HEAVEN')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("order temple check1")
			if pCity.isHolyCityByType(gc.getInfoTypeForString('RELIGION_THE_ORDER')):
				if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ORDER'):
					if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
						if not pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
							if not pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
								if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_ORDER')) == 0:
									if pCity.getBaseYieldRate(1) >=5:
										if pPlot.getNumUnits() >= 2:
											pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iOrderTemple, -1, False, False, False, False)
											bOverride = true
											print ("order temple built")
											return bOverride
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ORDERS_FROM_HEAVEN')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("order aco check1")
			if pCity.isHolyCityByType(gc.getInfoTypeForString('RELIGION_THE_ORDER')):
				if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ORDER'):
					if not pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
						if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
							if not pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
								if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_ORDER')) == 1:
									if pCity.getBaseYieldRate(1) >=5:
										if pPlot.getNumUnits() >= 2:
											pCity.pushOrder(OrderTypes.ORDER_TRAIN, iOrderDisciple, -1, False, False, False, False)
											bOverride = true
											print ("order disciple built")
											return bOverride
		#spread the empyrean if has holy city
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HONOR')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("empy temple check1")
			if pCity.isHolyCityByType(gc.getInfoTypeForString('RELIGION_THE_EMPYREAN')):
				if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_EMPYREAN'):
					if not pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
						if not pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
							if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
								if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_EMPYREAN')) == 0:
									if pCity.getBaseYieldRate(1) >=5:
										if pPlot.getNumUnits() >= 2:
											pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iEmpyreanTemple, -1, False, False, False, False)
											bOverride = true
											print ("empyrean temple built")
											return bOverride
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HONOR')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("empy disc check1")
			if pCity.isHolyCityByType(gc.getInfoTypeForString('RELIGION_THE_EMPYREAN')):
				if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_EMPYREAN'):
					if not pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
						if not pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
							if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
								if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_EMPYREAN')) == 1:
									if pCity.getBaseYieldRate(1) >=5:
										if pPlot.getNumUnits() >= 2:
											pCity.pushOrder(OrderTypes.ORDER_TRAIN, iEmpyreanDisciple, -1, False, False, False, False)
											bOverride = true
											print ("empyrean disciple built")
											return bOverride
		#spread esus if has holy city, spreading it is cheap, but don't produce nightwatches all day long
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_DECEPTION')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("esus disc check1")
			if pCity.isHolyCityByType(gc.getInfoTypeForString('RELIGION_COUNCIL_OF_ESUS')):
				if CyGame().getSorenRandNum(100, "Esusspread") <= 20:
					if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_COUNCIL_OF_ESUS'):
						if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
							if pCity.getBaseYieldRate(1) >=5:
								if pPlot.getNumUnits() >= 2:
									if pCity.canTrain(gc.getInfoTypeForString('UNIT_NIGHTWATCH'),False,False):
										pCity.pushOrder(OrderTypes.ORDER_TRAIN, (gc.getInfoTypeForString('UNIT_NIGHTWATCH')), -1, False, False, False, False)
										bOverride = true
										print ("nightwatch built")
										return bOverride
		#spread leaves if elven or has holy city
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("leaves temple check1")
			if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR') or pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR') or pCity.isHolyCityByType(gc.getInfoTypeForString('RELIGION_FELLOWSHIP_OF_LEAVES')):
				if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_FELLOWSHIP_OF_LEAVES'):
					if not pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
						if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
							if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_TEMPLE_OF_LEAVES')) == 0:
								if pCity.getBaseYieldRate(1) >=5:
									if pPlot.getNumUnits() >= 2:
										pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iLeavesTemple, -1, False, False, False, False)
										bOverride = true
										print ("leaves temple built")
										return bOverride
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("leaves disciple check1")
			if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR') or pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR') or pCity.isHolyCityByType(gc.getInfoTypeForString('RELIGION_FELLOWSHIP_OF_LEAVES')):
				if not pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_FELLOWSHIP_OF_LEAVES'):
					if not pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
						if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
							if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_TEMPLE_OF_LEAVES')) == 1:
								if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_FELLOWSHIP_OF_LEAVES')) <=3:
									if pCity.getBaseYieldRate(1) >=5:
										if pPlot.getNumUnits() >= 2:
											pCity.pushOrder(OrderTypes.ORDER_TRAIN, iLeavesDisciple, -1, False, False, False, False)
											bOverride = true
											print ("leaves disciple built")
											return bOverride
											
											
											
		#build the towers if possible
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ELEMENTALISM')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("ele tower check1")
			if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_TOWER_OF_THE_ELEMENTS')) == 0:
				if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_FIRE')):
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_WATER')):
						if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_AIR')):
							if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_EARTH')):
								if pCity.getBaseYieldRate(1) >=25:
									if pPlot.getNumUnits() >= 2:
										pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iTowerElement, -1, False, False, False, False)
										bOverride = true
										print ("element tower built")
										return bOverride
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_DIVINATION')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("div tower check1")
			if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_TOWER_OF_DIVINATION')) == 0:
				if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_SUN')):
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_SPIRIT')):
						if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_LAW')):
							if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_MIND')):
								if pCity.getBaseYieldRate(1) >=25:
									if pPlot.getNumUnits() >= 2:
										pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iTowerDivination, -1, False, False, False, False)
										bOverride = true
										print ("div tower built")
										return bOverride
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_NECROMANCY')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("tower necro check1")
			if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_TOWER_OF_NECROMANCY')) == 0:
				if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_SHADOW')):
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_DEATH')):
						if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_ENTROPY')):
							if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_CHAOS')):
								if pCity.getBaseYieldRate(1) >=25:
									if pPlot.getNumUnits() >= 2:
										pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iTowerNecromancy, -1, False, False, False, False)
										bOverride = true
										print ("necro tower built")
										return bOverride
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ALTERATION')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("tower alt check1")
			if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_TOWER_OF_ALTERATION')) == 0:
				if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_BODY')):
					if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_NATURE')):
						if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_LIFE')):
							if pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_ENCHANTMENT')):
								if pCity.getBaseYieldRate(1) >=25:
									if pPlot.getNumUnits() >= 2:
										pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iTowerAlteration, -1, False, False, False, False)
										bOverride = true
										print ("alteration tower built")
										return bOverride
		#prioritize the nat epic
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WARFARE')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_WRITING')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("lib epic check1")
			if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_NATIONAL_EPIC')) == 0:
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_LIBRARY')) == 0:
					if pCity.getBaseGreatPeopleRate() >=12:
						if pCity.getBaseYieldRate(1) >=10:
							if pPlot.getNumUnits() >= 2:
								if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_CLAN_OF_EMBERS'):
									if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_DOVIELLO'):
										pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iLibrary, -1, False, False, False, False)
										bOverride = true
										print ("nat epic library built")
										return bOverride
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WARFARE')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_WRITING')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("nat epic check1")
			if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_NATIONAL_EPIC')) == 0:
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_LIBRARY')) == 1:
					if pCity.getBaseGreatPeopleRate() >=12:
						if pCity.getBaseYieldRate(1) >=10:
							if pPlot.getNumUnits() >= 2:
								pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iNationalEpic, -1, False, False, False, False)
								bOverride = true
								print ("nat epic built")
								return bOverride
		#prioritize the heroic epic
		if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WARFARE')) and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("heroic epic check1")
			if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_HEROIC_EPIC')) == 0:
				if pCity.findBaseYieldRateRank(YieldTypes.YIELD_PRODUCTION)==1 and pCity.canConstruct(iHeroicEpic,False,False,False):
					if pCity.getBaseYieldRate(1) >=10:
						if pPlot.getNumUnits() >= 2:
							pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iHeroicEpic, -1, False, False, False, True)
							bOverride = true
							print ("heroic epic built")
							return bOverride
		#build the shrine in the he city
		if pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
			print ("championshrine check1")
			if pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_SHRINE_OF_THE_CHAMPION')) == 0:
				if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_HEROIC_EPIC')) == 1:
					if pPlot.getNumUnits() >= 2:
						if pCity.canConstruct(iShrineoftheChampion,False,False,False):
							pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iShrineoftheChampion, -1, False, False, False, True)
							bOverride = true
							print ("Shrine of the Champion built")
							return bOverride
		#Civ can build a Hero?
		if pCity.findBaseYieldRateRank(YieldTypes.YIELD_PRODUCTION)<=2:
			print ("Rank Hero Check1")
			if pCity.getBaseYieldRate(1) >=10:
				print ("Rank Hero Check2")
				sHero=['UNITCLASS_DONAL','UNITCLASS_TEUTORIX','UNITCLASS_CORLINDALE','UNITCLASS_BASIUM','UNITCLASS_GUYBRUSH']
				sHero=sHero+['UNITCLASS_EURABATRES','UNITCLASS_GILDEN','UNITCLASS_MAROS','UNITCLASS_MAGNADINE']
				sHero=sHero+['UNITCLASS_GOVANNON','UNITCLASS_LOKI','UNITCLASS_RANTINE','UNITCLASS_ALAZKAN']
				sHero=sHero+['UNITCLASS_LOSHA','UNITCLASS_ABASHI','UNITCLASS_RATHUS','UNITCLASS_WILBOMAN','UNITCLASS_HYBOREM']
				sHero=sHero+['UNITCLASS_YVAIN','UNITCLASS_KITHRA','UNITCLASS_BAMBUR','UNITCLASS_ARTHENDAIN']
				sHero=sHero+['UNITCLASS_SAVEROUS','UNITCLASS_HEMAH','UNITCLASS_ROSIER','UNITCLASS_MARDERO']
				sHero=sHero+['UNITCLASS_VALIN','UNITCLASS_SPHENER','UNITCLASS_CHALID','UNITCLASS_GIBBON','UNITCLASS_DUIN']
				for k in range(len(sHero)):
					iUnit = infoCiv.getCivilizationUnits(gc.getInfoTypeForString(sHero[k]))
					if iUnit != -1:
						if pCity.canTrain(iUnit,False,False):
							print iUnit
							pCity.pushOrder(OrderTypes.ORDER_TRAIN,iUnit,-1, False, False, False, True)			
							print ("Hero %s built"%(gc.getUnitInfo(iUnit).getType()))
							bOverride = true
							return bOverride
		# helping the Illians understand their temple
		if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_ILLIANS'):
			if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_HAND')) == 0:
				print ("Illian snowtemple check")
				iTemple = gc.getInfoTypeForString('BUILDING_TEMPLE_OF_THE_HAND')
				iFloodPlains = gc.getInfoTypeForString('FEATURE_FLOOD_PLAINS')
				iJungle = gc.getInfoTypeForString('FEATURE_JUNGLE')
				iScrub = gc.getInfoTypeForString('FEATURE_SCRUB')
				iTundra = gc.getInfoTypeForString('TERRAIN_TUNDRA')
				iDesert = gc.getInfoTypeForString('TERRAIN_DESERT')
				iSnowcounter = 0
				for iI in range(1, 21):
					pLoopPlot = pCity.getCityIndexPlot(iI)
					if pLoopPlot:
						print iSnowcounter
						if not pLoopPlot.isNone():
							if not pLoopPlot.isWater():
								if pLoopPlot.getTerrainType() == iTundra:
									iSnowcounter += 1
								if pLoopPlot.getTerrainType() == iDesert:
									iSnowcounter += 1
								iFeature = pLoopPlot.getFeatureType()
								if iFeature == iJungle:
									iSnowcounter += 1
								if iFeature == iFloodPlains:
									iSnowcounter -= 3
								if iFeature == iScrub:
									iSnowcounter += 1
								if pLoopPlot.isRiverSide():
									iSnowcounter -= 1
				if iSnowcounter>=1:
					if pPlot.getNumUnits() >= 2:
						if pCity.canConstruct(iTemple,False,False,False):
							pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, iTemple, -1, False, False, False, False)
							print ("Ice Temple built")
							bOverride = true
							return bOverride

		return bOverride
		
	def AI_unitUpdate(self,argsList):
		pUnit = argsList[0]
		pPlot = pUnit.plot()
		
		
			
		if pUnit.getUnitClassType() == gc.getInfoTypeForString('UNITCLASS_GIANT_SPIDER'):
			iX = pUnit.getX()
			iY = pUnit.getY()
			for iiX in range(iX-1, iX+2, 1):
				for iiY in range(iY-1, iY+2, 1):
					pLoopPlot = CyMap().plot(iiX,iiY)
					for i in range(pLoopPlot.getNumUnits()):
						if pLoopPlot.getUnit(i).getOwner() != pUnit.getOwner():
							return 0
			return 1

		if pUnit.getUnitType() == gc.getInfoTypeForString('UNIT_ACHERON'):
			if pPlot.isVisibleEnemyUnit(pUnit.getOwner()):
				pUnit.cast(gc.getInfoTypeForString('SPELL_BREATH_FIRE'))

		iImprovement = pPlot.getImprovementType()
		if iImprovement != -1:
			if (iImprovement == gc.getInfoTypeForString('IMPROVEMENT_BARROW') or iImprovement == gc.getInfoTypeForString('IMPROVEMENT_RUINS') or iImprovement == gc.getInfoTypeForString('IMPROVEMENT_HELLFIRE')):
				if not pUnit.isAnimal():
					if pPlot.getNumUnits() - pPlot.getNumAnimalUnits() == 1:
						return 1
			if (iImprovement == gc.getInfoTypeForString('IMPROVEMENT_BEAR_DEN') or iImprovement == gc.getInfoTypeForString('IMPROVEMENT_LION_DEN')):
				if pUnit.isAnimal():
					if pPlot.getNumAnimalUnits() == 1:
						return 1
			if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_GOBLIN_FORT'):
				if not pUnit.isAnimal():
					if pPlot.getNumUnits() - pPlot.getNumAnimalUnits() <= 3:
						return 1
		
		return False

	def AI_doWar(self,argsList):
		eTeam = argsList[0]
		return False

	def AI_doDiplo(self,argsList):
		ePlayer = argsList[0]
		return False

	def calculateScore(self,argsList):
		ePlayer = argsList[0]
		bFinal = argsList[1]
		bVictory = argsList[2]
		
		iPopulationScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getPopScore(), gc.getGame().getInitPopulation(), gc.getGame().getMaxPopulation(), gc.getDefineINT("SCORE_POPULATION_FACTOR"), True, bFinal, bVictory)
		iLandScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getLandScore(), gc.getGame().getInitLand(), gc.getGame().getMaxLand(), gc.getDefineINT("SCORE_LAND_FACTOR"), True, bFinal, bVictory)
		iTechScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getTechScore(), gc.getGame().getInitTech(), gc.getGame().getMaxTech(), gc.getDefineINT("SCORE_TECH_FACTOR"), True, bFinal, bVictory)
		iWondersScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getWondersScore(), gc.getGame().getInitWonders(), gc.getGame().getMaxWonders(), gc.getDefineINT("SCORE_WONDER_FACTOR"), False, bFinal, bVictory)
		return int(iPopulationScore + iLandScore + iWondersScore + iTechScore)

	def doHolyCity(self):
		return False

	def doHolyCityTech(self,argsList):
		eTeam = argsList[0]
		ePlayer = argsList[1]
		eTech = argsList[2]
		bFirst = argsList[3]
		return False

	def doGold(self,argsList):
		ePlayer = argsList[0]
		return False

	def doResearch(self,argsList):
		ePlayer = argsList[0]
		return False

	def doGoody(self,argsList):
		ePlayer = argsList[0]
		pPlot = argsList[1]
		pUnit = argsList[2]
		return False

	def doGrowth(self,argsList):
		pCity = argsList[0]
		return False

	def doProduction(self,argsList):
		pCity = argsList[0]
		return False

	def doCulture(self,argsList):
		pCity = argsList[0]
		return False

	def doPlotCulture(self,argsList):
		pCity = argsList[0]
		bUpdate = argsList[1]
		ePlayer = argsList[2]
		iCultureRate = argsList[3]
		return False

	def doReligion(self,argsList):
		pCity = argsList[0]
		return False

	def cannotSpreadReligion(self,argsList):
		iOwner, iUnitID, iReligion, iX, iY = argsList[0]
		return False

	def doGreatPeople(self,argsList):
		pCity = argsList[0]
		return False

	def doMeltdown(self,argsList):
		pCity = argsList[0]
		return False
	
	def doReviveActivePlayer(self,argsList):
		"allows you to perform an action after an AIAutoPlay"
		iPlayer = argsList[0]
		return False
	
	def doPillageGold(self, argsList):
		"controls the gold result of pillaging"
		pPlot = argsList[0]
		pUnit = argsList[1]
		
		iPillageGold = 0
		iPillageGold = CyGame().getSorenRandNum(gc.getImprovementInfo(pPlot.getImprovementType()).getPillageGold(), "Pillage Gold 1")
		iPillageGold += CyGame().getSorenRandNum(gc.getImprovementInfo(pPlot.getImprovementType()).getPillageGold(), "Pillage Gold 2")

		iPillageGold += (pUnit.getPillageChange() * iPillageGold) / 100
		
		return iPillageGold
	
	def doCityCaptureGold(self, argsList):
		"controls the gold result of capturing a city"
		
		pOldCity = argsList[0]
		
		iCaptureGold = 0
		
		iCaptureGold += gc.getDefineINT("BASE_CAPTURE_GOLD")
		iCaptureGold += (pOldCity.getPopulation() * gc.getDefineINT("CAPTURE_GOLD_PER_POPULATION"))
		iCaptureGold += CyGame().getSorenRandNum(gc.getDefineINT("CAPTURE_GOLD_RAND1"), "Capture Gold 1")
		iCaptureGold += CyGame().getSorenRandNum(gc.getDefineINT("CAPTURE_GOLD_RAND2"), "Capture Gold 2")

		if (gc.getDefineINT("CAPTURE_GOLD_MAX_TURNS") > 0):
			iCaptureGold *= cyIntRange((CyGame().getGameTurn() - pOldCity.getGameTurnAcquired()), 0, gc.getDefineINT("CAPTURE_GOLD_MAX_TURNS"))
			iCaptureGold /= gc.getDefineINT("CAPTURE_GOLD_MAX_TURNS")
		
		return iCaptureGold
	
	def citiesDestroyFeatures(self,argsList):
		iX, iY= argsList
		return True
		
	def canFoundCitiesOnWater(self,argsList):
		iX, iY= argsList
		return False
		
	def doCombat(self,argsList):
		pSelectionGroup, pDestPlot = argsList
		return False

	def getConscriptUnitType(self, argsList):
		iPlayer = argsList[0]
		iConscriptUnitType = -1 #return this with the value of the UNIT TYPE you want to be conscripted, -1 uses default system
		
		return iConscriptUnitType

	def getCityFoundValue(self, argsList): #gets called 4k+ times/turn,disabled
		iPlayer, iPlotX, iPlotY = argsList
		iFoundValue = -1 # Any value besides -1 will be used
		pPlot = CyMap().plot(iPlotX,iPlotY)
		pPlayer = gc.getPlayer(iPlayer)
		eTeam = gc.getTeam(pPlayer.getTeam())
		print ("performance call foundvalue")
		if pPlayer.isHuman() == False:
			if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KURIOTATES'):
				if pPlot.isWithinteamCityRadius(eTeam,pPlayer):
					iFoundValue= iFoundValue-10000
		return iFoundValue
		
	def canPickPlot(self, argsList):
		pPlot = argsList[0]
		return true
		
	def getUnitCostMod(self, argsList):
		iPlayer, iUnit = argsList
		iCostMod = -1 # Any value > 0 will be used
		
		return iCostMod

	def getBuildingCostMod(self, argsList):
		iPlayer, iCityID, iBuilding = argsList
		pPlayer = gc.getPlayer(iPlayer)
		pCity = pPlayer.getCity(iCityID)

		iCostMod = -1 # Any value > 0 will be used

		if iBuilding == gc.getInfoTypeForString('BUILDING_GAMBLING_HOUSE'):
			if pPlayer.isGamblingRing():
				iCostMod = gc.getBuildingInfo(iBuilding).getProductionCost() / 4
		
		return iCostMod
		
	def canUpgradeAnywhere(self, argsList):
		pUnit = argsList
		
		bCanUpgradeAnywhere = 0
		
		return bCanUpgradeAnywhere
		
	def getWidgetHelp(self, argsList):
		eWidgetType, iData1, iData2, bOption = argsList
		
		return u""
		
	def getUpgradePriceOverride(self, argsList):
		iPlayer, iUnitID, iUnitTypeUpgrade = argsList
		
		return -1	# Any value 0 or above will be used
	
	def getExperienceNeeded(self, argsList):
		# use this function to set how much experience a unit needs
		iLevel, iOwner = argsList
		
		iExperienceNeeded = 0

		# regular epic game experience		
		iExperienceNeeded = iLevel * iLevel + 1

		iModifier = gc.getPlayer(iOwner).getLevelExperienceModifier()
		if (0 != iModifier):
			iExperienceNeeded += (iExperienceNeeded * iModifier + 99) / 100   # ROUND UP
			
		return iExperienceNeeded
